@@ -4,7 +4,7 @@
  Copyright (C) 2004 Aaron Stone aaron at serendipity dot cx
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -204,8 +204,17 @@ void dsnuser_free(Delivery_T * dsnuser)
 	dsnuser->dsn.detail = 0;
 	dsnuser->source = BOX_NONE;
 
-	g_list_destroy(dsnuser->userids);
-	g_list_destroy(dsnuser->forwards);
+	if (dsnuser->userids) {
+		dsnuser->userids = g_list_first(dsnuser->userids);
+		g_list_free_full(g_steal_pointer (&dsnuser->userids), g_free);
+		dsnuser->userids = NULL;
+	}
+
+	if (dsnuser->forwards) {
+		dsnuser->forwards = g_list_first(dsnuser->forwards);
+		g_list_free_full(g_steal_pointer (&dsnuser->forwards), g_free);
+		dsnuser->forwards = NULL;
+	}
 
 	if (dsnuser->address) {
 		g_free(dsnuser->address);
@@ -215,7 +224,7 @@ void dsnuser_free(Delivery_T * dsnuser)
 		g_free(dsnuser->mailbox);
 		dsnuser->mailbox = NULL;
 	}
-	
+
 	TRACE(TRACE_DEBUG, "dsnuser freed");
 }
 
@@ -233,7 +242,7 @@ static int address_has_alias(Delivery_T *delivery)
 
 	if (!delivery->address)
 		return 0;
-
+	TRACE(TRACE_DEBUG, "address_has_alias [%s]", delivery->address);
 	alias_count = auth_check_user_ext(delivery->address, &delivery->userids, &delivery->forwards, 0);
 	TRACE(TRACE_DEBUG, "user [%s] found total of [%d] aliases", delivery->address, alias_count);
 
@@ -243,6 +252,8 @@ static int address_has_alias(Delivery_T *delivery)
 	return 0;
 }
 
+
+
 // address is in format username+mailbox@domain
 // and we want to cut out the mailbox and produce
 // an address in format username@domain, then check it.
@@ -346,7 +357,6 @@ static int address_is_domain_catchall(Delivery_T *delivery)
         
 		/* Checking for domain aliases */
 		domain_count = auth_check_user_ext(my_domain, &delivery->userids, &delivery->forwards, 0);
-        
 		if (domain_count > 0) {
 			/* This is the way to succeed out. */
 			break;
@@ -514,7 +524,10 @@ int dsnuser_resolve(Delivery_T *delivery)
 	uint64_t *uid;
 	/* If the userid is already set, then we're doing direct-to-userid.
 	 * We just want to make sure that the userid actually exists... */
-	if (delivery->useridnr != 0) {
+	if (! delivery) {
+		TRACE(TRACE_ERR, "This delivery had nothing to go on.");
+		return -1;
+	} else if (delivery->useridnr != 0) {
 
 		TRACE(TRACE_INFO, "checking if [%" PRIu64 "] is a valid useridnr.", delivery->useridnr);
 
