@@ -1,7 +1,7 @@
 /*
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -215,11 +215,19 @@ char * dbmail_iconv_db_to_utf7(const char* str_in)
 
 char * dbmail_iconv_decode_text(const char *in)
 {
-	return g_mime_utils_header_decode_text(NULL, in);
+	char * ret;
+	GMimeParserOptions *options = g_mime_parser_options_new ();
+	ret = g_mime_utils_header_decode_text(options, in);
+	g_mime_parser_options_free(options);
+	return ret;
 }
 char * dbmail_iconv_decode_address(char *address)
 {
-	return g_mime_utils_header_decode_phrase(NULL, address);
+	char * ret;
+	GMimeParserOptions *options = g_mime_parser_options_new ();
+	ret = g_mime_utils_header_decode_phrase(options, address);
+	g_mime_parser_options_free(options);
+	return ret;
 }
 
 char * dbmail_iconv_decode_field(const char *in, const char *charset, gboolean isaddr)
