@@ -1,7 +1,7 @@
 /*
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -179,6 +179,7 @@ static uint64_t dbmail_imap_session_message_load(ImapSession *self)
 			/* previous behavior, a query will be performed in db */
 			if ((db_get_physmessage_id(self->msg_idnr, id)) != DM_SUCCESS) {
 				TRACE(TRACE_ERR,"can't find physmessage_id for message_idnr [%" PRIu64 "]", self->msg_idnr);
+				mempool_push(self->pool, id, sizeof(uint64_t));
 				g_free(id);
 				return 0;
 			}
@@ -191,6 +192,7 @@ static uint64_t dbmail_imap_session_message_load(ImapSession *self)
 		
 	if (self->message) {
 		if (*id != self->message->id) {
+			TRACE(TRACE_DEBUG,"id != message->id [%" PRIu64 "] [%" PRIu64 "]", *id, self->message->id);
 			dbmail_message_free(self->message);
 			self->message = NULL;
 		}
@@ -716,7 +718,8 @@ void _send_headers(ImapSession *self, const body_fetch *bodyfetch, gboolean not)
 static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean not)
 {
 	Connection_T c; ResultSet_T r; volatile int t = FALSE;
-	gchar *fld, *val, *old, *new = NULL;
+	const gchar *fld;
+	gchar *fld2, *val, *old, *new = NULL;
 	uint64_t *mid;
 	uint64_t id;
 	GList *last;
@@ -744,7 +747,7 @@ static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean no
 			tlist = g_list_append(tlist, (void *)p_string_str(self->args[k + bodyfetch->argstart]));
 
 		bodyfetch->hdrplist = dbmail_imap_plist_as_string(tlist);
-		h = g_list_join((GList *)tlist,"','");
+		h = g_list_join(tlist,"','");
 		bodyfetch->names = tlist;
 
 		h = g_string_ascii_down(h);
@@ -790,12 +793,6 @@ static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean no
 			g_string_append_printf(fieldorder, "WHEN n.headername='%s' THEN %d ",
 					name, fieldseq);
 			
-			if (! g_list_next(bodyfetch->names)){
-				g_free(name);
-				break;
-			}
-			bodyfetch->names = g_list_next(bodyfetch->names);
-			fieldseq++;
 			/* get the id of the header */
 			query = p_string_new(self->pool, "");
 			p_string_printf(query, "select id from %sheadername "
@@ -818,8 +815,16 @@ static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean no
 			END_TRY;
 			p_string_free(query, TRUE);		
 			g_free(name);
+			if (! g_list_next(bodyfetch->names))
+				break;
+			bodyfetch->names = g_list_next(bodyfetch->names);
+			fieldseq++;
+		}
+		bodyfetch->names = g_list_first(bodyfetch->names);
+		if (g_list_length(bodyfetch->names)==0) {
+			// Avoid empty case error when there are no names
+			g_string_append_printf(fieldorder, "WHEN TRUE THEN 1 ");
 		}
-		fieldseq++;
 		//adding default value, useful in NOT conditions, Cosmin Cioranu
 		g_string_append_printf(fieldorder, "ELSE %d END AS seq",fieldseq);
 	}
@@ -861,11 +866,11 @@ static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean no
 			const void *blob;
 
 			id = db_result_get_u64(r, 0);
-			
+
 			if (! g_tree_lookup(self->ids,&id))
 				continue;
-			
-			fld = (char *)db_result_get(r, 1);
+
+			fld = db_result_get(r, 1);
 			blob = db_result_get_blob(r, 2, &l);
 			char *str = g_new0(char, l + 1);
 			str = strncpy(str, blob, l);
@@ -878,13 +883,15 @@ static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean no
 				*mid = id;
 
 				old = g_tree_lookup(bodyfetch->headers, (gconstpointer)mid);
-				fld[0] = toupper(fld[0]);
+
 				/* Build content as Header: value \n */
-				new = g_strdup_printf("%s%s: %s\n", old?old:"", fld, val);
+				fld2 = g_strdup_printf("%s", fld);
+				fld2[0] = toupper(fld[0]);
+				new = g_strdup_printf("%s%s: %s\n", old?old:"", fld2, val);
 				g_free(val);
+				g_free(fld2);
 				g_tree_insert(bodyfetch->headers,mid,new);
 			}
-			
 		}
 	CATCH(SQLException)
 		LOG_SQLERROR;
@@ -1152,7 +1159,7 @@ static int _fetch_get_items(ImapSession *self, uint64_t *uid)
 
 		GList *sublist = MailboxState_message_flags(self->mailbox->mbstate, msginfo);
 		s = dbmail_imap_plist_as_string(sublist);
-		g_list_destroy(sublist);
+		g_list_free_full(g_steal_pointer (&sublist), g_free);
 		dbmail_imap_session_buff_printf(self,"FLAGS %s",s);
 		g_free(s);
 	}
@@ -1462,8 +1469,8 @@ static void notify_fetch(ImapSession *self, MailboxState_T N, uint64_t *uid)
 	nl = MailboxState_message_flags(N, new);
 	newflags = dbmail_imap_plist_as_string(nl);
 
-	g_list_destroy(ol);
-	g_list_destroy(nl);
+	g_list_free_full(g_steal_pointer (&ol), g_free);
+	g_list_free_full(g_steal_pointer (&nl), g_free);
 
 	if ((!old) || (old->seq < new->seq))
 		modseqchanged = true;
@@ -1495,7 +1502,7 @@ static void notify_fetch(ImapSession *self, MailboxState_T N, uint64_t *uid)
 		dbmail_imap_session_buff_printf(self, "* %" PRIu64 " FETCH %s\r\n", 
 				*msn, response);
 		g_free(response);
-		g_list_destroy(plist);
+		g_list_free_full(g_steal_pointer (&plist), g_free);
 	}
 
 	if (oldflags) g_free(oldflags);
@@ -1717,6 +1724,7 @@ MailboxState_T dbmail_imap_session_mbxinfo_lookup(ImapSession *self, uint64_t ma
 {
 	MailboxState_T M = NULL;
 	uint64_t *id;
+	uint64_t oldexists = 0, newexists = 0;
 	if (self->mailbox && self->mailbox->mbstate && (MailboxState_getId(self->mailbox->mbstate) == mailbox_id)) {
 		// selected state
 		M = self->mailbox->mbstate;
@@ -1729,7 +1737,6 @@ MailboxState_T dbmail_imap_session_mbxinfo_lookup(ImapSession *self, uint64_t ma
 			g_tree_replace(self->mbxinfo, id, M);
 		} else {
 			unsigned newseq = 0, oldseq = 0;
-			unsigned newexists = 0, oldexists = 0;
 			MailboxState_T N = NULL;
 			N = MailboxState_new(self->pool, 0);
 			MailboxState_setId(N, mailbox_id);
@@ -1738,6 +1745,7 @@ MailboxState_T dbmail_imap_session_mbxinfo_lookup(ImapSession *self, uint64_t ma
 			oldexists = MailboxState_getExists(M);
 			MailboxState_free(&N);
 			if (oldseq < newseq) {
+				TRACE(TRACE_DEBUG,"oldseq/newseq [%d]/[%d]", oldseq, newseq);
 				id = mempool_pop(small_pool, sizeof(uint64_t));
 				*id = mailbox_id;
 				M = MailboxState_new(self->pool, mailbox_id);
@@ -1746,8 +1754,6 @@ MailboxState_T dbmail_imap_session_mbxinfo_lookup(ImapSession *self, uint64_t ma
 				g_tree_replace(self->mbxinfo, id, M);
 			}
 		}
-
-
 	}
 
 	assert(M);
@@ -1802,7 +1808,9 @@ int dbmail_imap_session_set_state(ImapSession *self, ClientState_T state)
 static gboolean _do_expunge(uint64_t *id, ImapSession *self)
 {
 	MessageInfo *msginfo = g_tree_lookup(MailboxState_getMsginfo(self->mailbox->mbstate), id);
-	assert(msginfo);
+	if (! msginfo) {
+		return FALSE;
+	}
 
 	if (! msginfo->flags[IMAP_FLAG_DELETED]) return FALSE;
 
@@ -1870,7 +1878,7 @@ static void _body_fetch_free(body_fetch *bodyfetch, gpointer data)
 	ImapSession *self = (ImapSession *)data;
 	if (! bodyfetch) return;
 	if (bodyfetch->names) {
-		g_list_free(g_list_first(bodyfetch->names));
+		g_list_destroy(bodyfetch->names);
 		bodyfetch->names = NULL;
 	}
 
@@ -1890,6 +1898,8 @@ void dbmail_imap_session_bodyfetch_free(ImapSession *self)
 	while (head) {
 		body_fetch *bodyfetch = p_list_data(head);
 		_body_fetch_free(bodyfetch, self);
+		if (!p_list_next(head))
+			break;
 		head = p_list_next(head);
 	}
 	p_list_free(&first);
@@ -1934,7 +1944,7 @@ void dbmail_imap_session_bodyfetch_free(ImapSession *self)
 int imap4_tokenizer_main(ImapSession *self, const char *buffer)
 {
 	int inquote = 0, quotestart = 0;
-	int nnorm = 0, nsquare = 0, paridx = 0, argstart = 0;
+	int paridx = 0, argstart = 0;
 	unsigned int i = 0;
 	size_t max;
 	char parlist[MAX_LINESIZE];
@@ -2045,7 +2055,6 @@ int imap4_tokenizer_main(ImapSession *self, const char *buffer)
 				if (paridx < 0 || parlist[paridx] != NORMPAR)
 					paridx = -1;
 				else {
-					nnorm--;
 					paridx--;
 				}
 
@@ -2057,7 +2066,6 @@ int imap4_tokenizer_main(ImapSession *self, const char *buffer)
 					paridx = -1;
 				else {
 					paridx--;
-					nsquare--;
 				}
 
 				break;
@@ -2065,14 +2073,12 @@ int imap4_tokenizer_main(ImapSession *self, const char *buffer)
 				case '(':
 				
 				parlist[++paridx] = NORMPAR;
-				nnorm++;
 				
 				break;
 
 				case '[':
 				
 				parlist[++paridx] = SQUAREPAR;
-				nsquare++;
 
 				break;
 			}
