@@ -1,7 +1,7 @@
 /*
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
@@ -25,32 +25,12 @@
  */
 
 #include "dbmail.h"
-//#include "gmime/gmime-header.h"
-//#include "gmime/gmime-message.h"
-//#include "gmime/gmime-object.h"
-//#include "gmime/gmime-stream.h"
-//#include "gmime/gmime-stream-mem.h"
 
 extern DBParam_T db_params;
 #define DBPFX db_params.pfx
 #define DBMAIL_TEMPMBOX "INBOX"
 #define THIS_MODULE "message"
 
-
-/*
- * used for debugging message de/re-construction
- */
-#ifdef DEBUG_MESSAGE
-#define dprint(fmt, args...) TRACE(TRACE_DEBUG, fmt, ##args); printf(fmt, ##args)
-#endif
-
-#ifndef dprint
-#define DPRINT 0
-#define dprint(fmt, args...) 
-#else
-#define DPRINT 1
-#endif
-
 static void _header_cache(const char *, const char *, gpointer);
 
 static gboolean _header_insert(uint64_t physmessage_id, uint64_t headername_id, uint64_t headervalue_id);
@@ -119,8 +99,13 @@ static uint64_t blob_exists(const char *buf, const char *hash)
 {
 	volatile uint64_t id = 0;
 	volatile uint64_t id_old = 0;
+	int message_part_hash=config_get_value_default_int("message_part_hash","DBMAIL",0);
 	size_t l;
 	assert(buf);
+	TRACE(TRACE_INFO,"mimeparts hash evaluation message_part_hash = %d value size [%lu]",message_part_hash,strlen(buf));
+	//no hash
+	if (message_part_hash==2)
+		return id;
 	Connection_T c; PreparedStatement_T s; ResultSet_T r;
 	char blob_cmp[DEF_FRAGSIZE];
 	memset(blob_cmp, 0, sizeof(blob_cmp));
@@ -142,10 +127,20 @@ static uint64_t blob_exists(const char *buf, const char *hash)
 			db_stmt_set_int(s, 3, l);
 			db_stmt_exec(s);
 			id = db_get_pk(c, "mimeparts");
-			s = db_stmt_prepare(c, "SELECT a.id, b.id FROM dbmail_mimeparts a INNER JOIN " 
-					"%smimeparts b ON a.hash=b.hash AND DBMS_LOB.COMPARE(a.data, b.data) = 0 " 
-					" AND a.id<>b.id AND b.id=?", DBPFX);
-			db_stmt_set_u64(s, 1, id);
+			//for oracle the optimization of message part hash are not implemented
+			switch(message_part_hash){
+			case 0:
+			case 1:
+				s = db_stmt_prepare(c, "SELECT a.id, b.id FROM dbmail_mimeparts a INNER JOIN "
+									"%smimeparts b ON a.hash=b.hash AND DBMS_LOB.COMPARE(a.data, b.data) = 0 "
+									" AND a.id<>b.id AND b.id=?", DBPFX);
+
+
+				db_stmt_set_u64(s, 1, id);
+				break;
+			default:
+				TRACE(TRACE_ERR,"unknown message_part_hash = %d value",message_part_hash);
+			}
 			r = db_stmt_query(s);
 			if (db_result_next(r))
 				id_old = db_result_get_u64(r,0);			
@@ -157,13 +152,27 @@ static uint64_t blob_exists(const char *buf, const char *hash)
 				db_commit_transaction(c);
 			}
 		} else {
-			snprintf(blob_cmp, DEF_FRAGSIZE-1, db_get_sql(SQL_COMPARE_BLOB), "data");
-			s = db_stmt_prepare(c,"SELECT id FROM %smimeparts WHERE hash=? AND %ssize%s=? AND %s", 
-					DBPFX,db_get_sql(SQL_ESCAPE_COLUMN), db_get_sql(SQL_ESCAPE_COLUMN),
-					blob_cmp);
-			db_stmt_set_str(s,1,hash);
-			db_stmt_set_u64(s,2,l);
-			db_stmt_set_blob(s,3,buf,l);
+			switch(message_part_hash){
+			case 0:
+				snprintf(blob_cmp, DEF_FRAGSIZE-1, db_get_sql(SQL_COMPARE_BLOB), "data");
+				s = db_stmt_prepare(c,"SELECT id FROM %smimeparts WHERE hash=? AND %ssize%s=? AND %s limit 1",
+						DBPFX,db_get_sql(SQL_ESCAPE_COLUMN), db_get_sql(SQL_ESCAPE_COLUMN),
+						blob_cmp);
+				db_stmt_set_str(s,1,hash);
+				db_stmt_set_u64(s,2,l);
+				db_stmt_set_blob(s,3,buf,l);
+				break;
+			case 1:
+				s = db_stmt_prepare(c,"SELECT id FROM %smimeparts WHERE hash=? AND %ssize%s=? limit 1",
+						DBPFX,db_get_sql(SQL_ESCAPE_COLUMN), db_get_sql(SQL_ESCAPE_COLUMN)
+					);
+				db_stmt_set_str(s,1,hash);
+				db_stmt_set_u64(s,2,l);
+				break;
+			default:
+				TRACE(TRACE_ERR,"unknown message_part_hash = %d value",message_part_hash);
+			}
+
 			r = db_stmt_query(s);
 			if (db_result_next(r))
 				id = db_result_get_u64(r,0);
@@ -186,15 +195,18 @@ static uint64_t blob_insert(const char *buf, const char *hash)
 	volatile uint64_t id = 0;
 	char *frag = db_returning("id");
 
+	TRACE(TRACE_DEBUG, "final blob store size [%lu] hash %s", strlen(buf),hash);
+
 	assert(buf);
 	l = strlen(buf);
 
 	c = db_con_get();
 	TRY
 		db_begin_transaction(c);
-		s = db_stmt_prepare(c, "INSERT INTO %smimeparts (hash, data, %ssize%s) VALUES (?, ?, ?) %s", 
+		s = db_stmt_prepare(c, "INSERT INTO %smimeparts (hash, data, %ssize%s) VALUES (?, ?, ?) %s",
 				DBPFX, db_get_sql(SQL_ESCAPE_COLUMN), db_get_sql(SQL_ESCAPE_COLUMN), frag);
 		db_stmt_set_str(s, 1, hash);
+
 		db_stmt_set_blob(s, 2, buf, l);
 		db_stmt_set_int(s, 3, l);
 		if (db_params.db_driver == DM_DRIVER_ORACLE) {
@@ -274,12 +286,14 @@ static int store_blob(DbmailMessage *m, const char *buf, gboolean is_header)
 
 	if (! buf) return 0;
 
+	TRACE(TRACE_DEBUG, "blob store size [%lu]", strlen(buf));
+
 	if (is_header) {
 		m->part_key++;
 		m->part_order=0;
 	}
 
-	dprint("<blob is_header=\"%d\" part_depth=\"%d\" part_key=\"%d\" part_order=\"%d\">\n%s\n</blob>\n", 
+	TRACE(TRACE_DEBUG, "<blob is_header=\"%d\" part_depth=\"%d\" part_key=\"%d\" part_order=\"%d\">\n%s\n</blob>\n",
 			is_header, m->part_depth, m->part_key, m->part_order, buf);
 
 	if (! (id = blob_store(buf)))
@@ -368,7 +382,7 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 	GMimeContentType *mimetype = NULL;
 	volatile int prevdepth, depth = 0, row = 0;
 	volatile int t = FALSE;
-	volatile gboolean got_boundary = FALSE, prev_boundary = FALSE, is_header = TRUE, prev_header, finalized=FALSE;
+	volatile gboolean got_boundary = FALSE, prev_boundary = FALSE, is_header = TRUE, prev_header;
 	volatile gboolean prev_is_message = FALSE, is_message = FALSE;
 	volatile String_T m = NULL, n = NULL;
 	const void *blob;
@@ -402,16 +416,12 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 		row = 0;
 		while (db_result_next(r)) {
 			int l;
-#if DPRINT
 			int order;
 			int key;
-#endif
 
 			prevdepth	= depth;
 			prev_header	= is_header;
-#if DPRINT
 			key		= db_result_get_int(r,0);
-#endif
 			depth		= db_result_get_int(r,1);
 			if (depth > MAX_MIME_DEPTH) {
 				TRACE(TRACE_WARNING, "MIME part depth exceeds allowed maximum [%d]",
@@ -419,9 +429,7 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 				continue;
 			}
 
-#if DPRINT
 			order		= db_result_get_int(r,2);
-#endif
 			is_header	= db_result_get_bool(r,3);
 			if (row == 0) {
 				memset(internal_date, 0, sizeof(internal_date));
@@ -444,16 +452,15 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 
 			if (is_header && find_boundary(str, &boundary[0])) {
 				got_boundary = TRUE;
-				dprint("<boundary depth=\"%d\">%s</boundary>\n", depth, boundary);
+				TRACE(TRACE_DEBUG, "<boundary depth=\"%d\">%s</boundary>\n", depth, boundary);
 				strncpy(blist[depth], boundary, MAX_MIME_BLEN-1);
 			}
 
 			while ((prevdepth > 0) && (prevdepth-1 >= depth) && blist[prevdepth-1][0]) {
-				dprint("\n--%s at %d -> %d--\n", blist[prevdepth-1], prevdepth, prevdepth-1);
+				TRACE(TRACE_DEBUG, "\n--%s at %d -> %d--\n", blist[prevdepth-1], prevdepth, prevdepth-1);
 				p_string_append_printf(m, "\n--%s--\n", blist[prevdepth-1]);
 				memset(blist[prevdepth-1], 0, MAX_MIME_BLEN);
 				prevdepth--;
-				finalized=TRUE;
 			}
 
 			if ((depth > 0) && (blist[depth-1][0]))
@@ -461,16 +468,16 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 
 			if (is_header){
 				if (prev_header && depth>0 && !prev_is_message) {
-					dprint("--%s\n", boundary);
+					TRACE(TRACE_DEBUG, "--%s\n", boundary);
 					p_string_append_printf(m, "--%s\n", boundary);
 				}else if (!prev_header || prev_boundary) {
-					dprint("\n--%s\n", boundary);
+					TRACE(TRACE_DEBUG, "\n--%s\n", boundary);
 					p_string_append_printf(m, "\n--%s\n", boundary);
 				}
 			}
 
 			p_string_append_printf(m, "%s", str);
-			dprint("<part is_header=\"%d\" depth=\"%d\" key=\"%d\" order=\"%d\">\n%s\n</part>\n", 
+			TRACE(TRACE_DEBUG, "<part is_header=\"%d\" depth=\"%d\" key=\"%d\" order=\"%d\">\n%s\n</part>\n",
 				is_header, depth, key, order, str);
 
 			if (is_header)
@@ -480,13 +487,12 @@ static DbmailMessage * _mime_retrieve(DbmailMessage *self)
 			row++;
 		}
 
-		if (row > 2 && boundary[0] && !finalized) {
-			dprint("\n--%s-- final\n", boundary);
-			p_string_append_printf(m, "\n--%s--\n", boundary);
-			finalized=1;
+		// Add final boundary delimiter line if required
+		if (row > 2 && blist[0][0]) {
+			TRACE(TRACE_DEBUG, "\n--%s-- final\n", blist[0]);
+			p_string_append_printf(m, "\n--%s--\n", blist[0]);
 		}
 
-
 	CATCH(SQLException)
 		LOG_SQLERROR;
 		t = DM_EQUERY;
@@ -523,6 +529,7 @@ static int store_body(GMimeObject *object, DbmailMessage *m)
 	int r;
 	char *text = g_mime_object_get_body(object);
 	if (! text) return 0;
+	if (strlen(text)==0) return 0;
 	r = store_blob(m, text, 0);
 	g_free(text);
 	return r;
@@ -540,7 +547,7 @@ static gboolean store_mime_multipart(GMimeObject *object, DbmailMessage *m, cons
 {
 	const char *boundary;
 	const char *preface = NULL, *postface = NULL;
-	int n = 0, i, c;
+	int i, c;
 
 	g_return_val_if_fail(GMIME_IS_OBJECT(object), TRUE);
 
@@ -555,7 +562,6 @@ static gboolean store_mime_multipart(GMimeObject *object, DbmailMessage *m, cons
 
 	if (boundary) {
 		m->part_depth++;
-		n = m->part_order;
 		m->part_order=0;
 	}
 
@@ -563,13 +569,11 @@ static gboolean store_mime_multipart(GMimeObject *object, DbmailMessage *m, cons
     gboolean store_mime_object_result;
 	for (i=0; i<c; i++) {
 		GMimeObject *part = g_mime_multipart_get_part((GMimeMultipart *)object, i);
-		// if (store_mime_object(object, part, m)) return TRUE;
 		store_mime_object_result = store_mime_object(object, part, m);
 		TRACE(TRACE_DEBUG,"store_mime_object: [%d] [%d]", i, store_mime_object_result);
 	}
 
 	if (boundary) {
-		n++;
 		m->part_depth--;
 		m->part_order++;
 	}
@@ -610,7 +614,7 @@ gboolean store_mime_object(GMimeObject *parent, GMimeObject *object, DbmailMessa
 	g_return_val_if_fail(GMIME_IS_OBJECT(object), TRUE);
 
 	if (GMIME_IS_MESSAGE(object)) {
-		dprint("\n<message>\n");
+		TRACE(TRACE_DEBUG, "\n<message>\n");
 
 		if(store_head(object,m) < 0) return TRUE;
 
@@ -643,7 +647,7 @@ gboolean store_mime_object(GMimeObject *parent, GMimeObject *object, DbmailMessa
 	}
 
 	if (GMIME_IS_MESSAGE(object)) {
-		dprint("\n</message>\n");
+		TRACE(TRACE_DEBUG, "\n</message>\n");
 	}
 
 	return r;
@@ -720,6 +724,7 @@ void dbmail_message_free(DbmailMessage *self)
 		return;
 
 	if (self->content) {
+		g_object_unref(self->content);
 		self->content = NULL;
 	}
 
@@ -788,6 +793,7 @@ DbmailMessage * dbmail_message_init_with_string(DbmailMessage *self, const char
 #define FROMLINE 80
 	char from[FROMLINE];
 	size_t buflen = strlen(str);
+	Field_T allow_invalid_messages = "no";
 
 	assert(self->content == NULL);
 
@@ -808,6 +814,8 @@ DbmailMessage * dbmail_message_init_with_string(DbmailMessage *self, const char
 		}
 	}
 
+	TRACE(TRACE_DEBUG, "Init messsage size [%lu] ", buflen);
+
 	self->stream = g_mime_stream_mem_new();
 	g_mime_stream_write(self->stream, str, buflen);
 	g_mime_stream_reset(self->stream);
@@ -822,7 +830,9 @@ DbmailMessage * dbmail_message_init_with_string(DbmailMessage *self, const char
 		self->content = content;
 		if (from[0])
 			dbmail_message_set_internal_date(self, from);
-	} else {
+	}
+	if (!content) {
+		TRACE(TRACE_DEBUG, "Messsage parse failed, trying to construct part");
 		content = GMIME_OBJECT(g_mime_parser_construct_part(parser, NULL));
 		g_object_unref(parser);
 		if (content) {
@@ -830,6 +840,45 @@ DbmailMessage * dbmail_message_init_with_string(DbmailMessage *self, const char
 			self->content = content;
 		}
 	}
+	config_get_value("allow_invalid_messages", "DBMAIL", allow_invalid_messages);
+	if (!content && strcmp(allow_invalid_messages, "yes") == 0) {
+		/* MIME part is invalid so add a simple text/plain mime header */
+		TRACE(TRACE_INFO, "Messsage parse part failed, converting to text/plain [%ld] offset [%d]",
+			self->internal_date,
+			self->internal_date_gmtoff);
+
+		GString *fixdata = NULL;
+		fixdata = g_string_new("MIME-Version: 1.0\r\nContent-Type: text/plain\r\n\r\n");
+		g_string_append(fixdata, str);
+
+		// Re-create the message part
+		g_object_unref(self->stream);
+		self->stream = g_mime_stream_mem_new();
+		g_mime_stream_write(self->stream, fixdata->str, fixdata->len);
+		g_mime_stream_reset(self->stream);
+
+		parser = g_mime_parser_new_with_stream(self->stream);
+
+		content = GMIME_OBJECT(g_mime_parser_construct_message(parser, NULL));
+		g_object_unref(parser);
+
+		if (content) {
+				dbmail_message_set_class(self, DBMAIL_MESSAGE);
+				self->content = content;
+				if (from[0]) {
+						dbmail_message_set_internal_date(self, from);
+				}
+		} else {
+			qprintf("Unable to convert to text/plain [%ld] offset [%d].\n",
+				self->internal_date,
+				self->internal_date_gmtoff);
+			TRACE(TRACE_ERR, "Unable to convert to text/plain [%ld] offset [%d]",
+				self->internal_date,
+				self->internal_date_gmtoff);
+		}
+
+		g_free(fixdata);
+	}
 
 	buf = dbmail_message_to_string(self);
 	crlf = get_crlf_encoded(buf);
@@ -859,6 +908,7 @@ void dbmail_message_set_internal_date(DbmailMessage *self, const char *internal_
 		if ((gdt = g_mime_utils_header_decode_date(internal_date))) {
 			dt = g_date_time_to_unix(gdt);
 			self->internal_date = dt;
+			g_date_time_unref(gdt);
 		}
 		TRACE(TRACE_DEBUG, "internal_date [%s] [%ld] offset [%d]",
 				internal_date,
@@ -872,10 +922,16 @@ gchar * dbmail_message_get_internal_date(const DbmailMessage *self, int thisyear
 {
 	char *res;
 	struct tm gmt;
-	assert(self->internal_date);
+	time_t dt;
+
 	
 	memset(&gmt,'\0', sizeof(struct tm));
-	gmtime_r(&self->internal_date, &gmt);
+	if (! self->internal_date) {
+		dt = g_date_time_to_unix(NULL);
+		gmtime_r(&dt, &gmt);
+	} else {
+		gmtime_r(&self->internal_date, &gmt);
+	}
 
 	/* override if the date is not sane */
 	if (thisyear && ((gmt.tm_year + 1900) > (thisyear + 1)))
@@ -1577,6 +1633,7 @@ static int _header_value_get_id(const char *value, const char *sortfield, const
 		return FALSE;
 
 	c = db_con_get();
+	// Test if header already inserted
 	TRY
 		db_begin_transaction(c);
 		if ((tmp = _header_value_exists(c, value, (const char *)hash)) != 0)
@@ -1599,10 +1656,45 @@ static gboolean _header_insert(uint64_t physmessage_id, uint64_t headername_id,
 {
 
 	Connection_T c; PreparedStatement_T s; volatile gboolean t = TRUE;
+	ResultSet_T r;
+	volatile uint64_t header_found = 0;
 
 	TRACE(TRACE_DEBUG, "Inserting header: [%lu] [%lu] [%lu]", physmessage_id, headername_id, headervalue_id);
 	c = db_con_get();
 	db_con_clear(c);
+
+	// Check if the header exists
+	TRY
+		db_begin_transaction(c);
+		s = db_stmt_prepare(c, "SELECT count(*) "
+			"FROM %sheader "
+			"WHERE physmessage_id = ? "
+			"AND headername_id = ? "
+			"AND headervalue_id = ?", DBPFX);
+		db_stmt_set_u64(s, 1, physmessage_id);
+		db_stmt_set_u64(s, 2, headername_id);
+		db_stmt_set_u64(s, 3, headervalue_id);
+		r = db_stmt_query(s);
+		while (ResultSet_next(r)) {
+			header_found = db_result_get_u64(r, 0);
+		}
+		db_commit_transaction(c);
+	CATCH(SQLException)
+		LOG_SQLWARNING;
+		db_rollback_transaction(c);
+	FINALLY
+		db_con_close(c);
+	END_TRY;
+
+	// Return if the header exists
+	if (header_found == 1) {
+		TRACE(TRACE_INFO, "Header already inserted: [%lu] [%lu] [%lu]", physmessage_id, headername_id, headervalue_id);
+		return t;
+	}
+
+	c = db_con_get();
+	db_con_clear(c);
+
 	TRY
 		db_begin_transaction(c);
 		s = db_stmt_prepare(c, "INSERT INTO %sheader (physmessage_id, headername_id, headervalue_id) VALUES (?,?,?)", DBPFX);
@@ -1785,6 +1877,7 @@ static void _header_cache(const char *header, const char *raw, gpointer user_dat
 		TRACE(TRACE_DEBUG,"date_fmt [%s]", date_fmt);
 		if (date_fmt) {
 			g_utf8_strncpy(sortfield, date_fmt, CACHE_WIDTH-1);
+			g_free(date_fmt);
 			TRACE(TRACE_DEBUG,"sortfield [%s]", sortfield);
 			date_fmt = g_date_time_format(date, "%Y-%m-%d");
 			g_utf8_strncpy(datefield, date_fmt, CACHE_WIDTH-1);
@@ -1849,7 +1942,7 @@ static void insert_field_cache(uint64_t physid, const char *field, const char *v
 
 void dbmail_message_cache_referencesfield(const DbmailMessage *self)
 {
-	GMimeReferences *refs, *head;
+	GMimeReferences *refs;
 	GTree *tree;
 	const char *referencesfield, *inreplytofield;
 	char *field;
@@ -1867,7 +1960,6 @@ void dbmail_message_cache_referencesfield(const DbmailMessage *self)
 		return;
 	}
 
-	head = refs;
 	tree = g_tree_new_full((GCompareDataFunc)dm_strcmpdata, NULL, NULL, NULL);
 
 	int refs_len = g_mime_references_length(refs);
@@ -1884,7 +1976,7 @@ void dbmail_message_cache_referencesfield(const DbmailMessage *self)
 	}
 
 	g_tree_destroy(tree);
-	g_mime_references_clear(head);
+	g_mime_references_free(refs);
 }
 
 void dbmail_message_cache_envelope(const DbmailMessage *self)
@@ -1903,9 +1995,9 @@ void dbmail_message_cache_envelope(const DbmailMessage *self)
 		db_stmt_exec(s);
 		db_commit_transaction(c);
 	CATCH(SQLException)
-		LOG_SQLERROR;
+		LOG_SQLWARNING;
 		db_rollback_transaction(c);
-		TRACE(TRACE_ERR, "insert envelope failed [%s]", envelope);
+		TRACE(TRACE_WARNING, "insert envelope failed [%s]", envelope);
 	FINALLY
 		db_con_close(c);
 	END_TRY;
@@ -2245,7 +2337,7 @@ dsn_class_t sort_deliver_to_mailbox(DbmailMessage *message,
 	}
 
 	// Ok, we have the ACL right, time to deliver the message.
-	switch (db_copymsg(message->msg_idnr, mboxidnr, useridnr, &newmsgidnr, TRUE)) {
+	switch (db_copymsg(message->msg_idnr, mboxidnr, useridnr, &newmsgidnr)) {
 	case -2:
 		TRACE(TRACE_ERR, "error copying message to user [%" PRIu64 "],"
 				"maxmail exceeded", useridnr);
@@ -2296,6 +2388,162 @@ static int parse_and_escape(const char *in, char **out)
 
 	return 0;
 }
+
+/* struct to manage cURL upload status */
+struct upload_status {
+  size_t bytes_read;
+};
+
+static char *payload_text;
+
+/* Used by send_smtpmail to upload email to cURL */
+static size_t payload_source(char *ptr, size_t size, size_t nmemb, void *userp)
+{
+  struct upload_status *upload_ctx = (struct upload_status *)userp;
+  const char *data;
+  size_t room = size * nmemb;
+
+  if((size == 0) || (nmemb == 0) || ((size*nmemb) < 1)) {
+    return 0;
+  }
+
+  data = &payload_text[upload_ctx->bytes_read];
+
+  if(data) {
+    size_t len = strlen(data);
+    if(room < len)
+      len = room;
+    memcpy(ptr, data, len);
+    upload_ctx->bytes_read += len;
+
+    return len;
+  }
+
+  return 0;
+}
+
+/* Send SMTP mail via cURL
+ * https://curl.se/libcurl/c/smtp-mail.html */
+int send_smtpmail(DbmailMessage *message,
+		const char *to,
+		const char *from)
+{
+	CURL *curl;
+	CURLcode res = CURLE_OK;
+	struct curl_slist *recipients = NULL;
+	char errbuf[CURL_ERROR_SIZE];
+	struct upload_status upload_ctx = { 0 };
+	Field_T smtp_host;
+	Field_T smtp_user;
+	Field_T smtp_password;
+
+	if (config_get_value("SMTP_HOST", "DBMAIL", smtp_host) < 0){
+		TRACE(TRACE_ERR, "no config value for smtp_host");
+		return -1;
+	}
+	TRACE(TRACE_DEBUG, "Using libcurl for smtp");
+
+	// This must be the synchronous libcurl easy interface
+	// to ensure atomic delivery or non delivery.
+	curl = curl_easy_init();
+	if(curl) {
+		res = curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errbuf);
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "Unable to CURLOPT_ERRORBUFFER");
+			TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+		errbuf[0] = 0;
+		/* This is the URL for your mailserver */
+		res = curl_easy_setopt(curl, CURLOPT_URL, smtp_host);
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "Unable to CURLOPT_URL");
+			TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+		config_get_value("SMTP_USER", "DBMAIL", smtp_user);
+		if (strlen(smtp_user) > 0 )  {
+			res = curl_easy_setopt(curl, CURLOPT_USERNAME, smtp_user);
+			if(res != CURLE_OK) {
+				TRACE(TRACE_ERR, "Unable to CURLOPT_USERNAME");
+				TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+					res, curl_easy_strerror(res), errbuf);
+			}
+		}
+		config_get_value("SMTP_PASSWORD", "DBMAIL", smtp_password);
+		if (strlen(smtp_password) >0 ) {
+			res = curl_easy_setopt(curl, CURLOPT_PASSWORD, smtp_password);
+			if(res != CURLE_OK) {
+				TRACE(TRACE_ERR, "Unable to CURLOPT_PASSWORD");
+				TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+					res, curl_easy_strerror(res), errbuf);
+			}
+		}
+		res = curl_easy_setopt(curl, CURLOPT_MAIL_FROM, from);
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "Unable to CURLOPT_MAIL_FROM");
+			TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+		recipients = curl_slist_append(recipients, to);
+		res = curl_easy_setopt(curl, CURLOPT_MAIL_RCPT, recipients);
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "Unable to CURLOPT_MAIL_RCPT");
+			TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+		payload_text = dbmail_message_to_string(message);
+		res = curl_easy_setopt(curl, CURLOPT_READFUNCTION, payload_source);
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "Unable to CURLOPT_READFUNCTION");
+			TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+		res = curl_easy_setopt(curl, CURLOPT_READDATA, &upload_ctx);
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "Unable to CURLOPT_READDATA");
+			TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+		res = curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "Unable to CURLOPT_UPLOAD");
+			TRACE(TRACE_ERR, "curl_easy_setopt: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+
+		/* Send the message */
+		res = curl_easy_perform(curl);
+
+		/* Check for errors */
+		if(res != CURLE_OK) {
+			TRACE(TRACE_ERR, "cURL smtp send failed: [%d] [%s] [%s]",
+				res, curl_easy_strerror(res), errbuf);
+		}
+
+		/* Free the list of recipients */
+		curl_slist_free_all(recipients);
+
+		/* curl does not send the QUIT command until you call cleanup, so you
+		* should be able to reuse this connection for additional messages
+		* (setting CURLOPT_MAIL_FROM and CURLOPT_MAIL_RCPT as required, and
+		* calling curl_easy_perform() again. It may not be a good idea to keep
+		* the connection open for a long time though (more than a few minutes may
+		* result in the server timing out the connection), and you do want to
+		* clean up in the end.
+		*/
+		curl_easy_cleanup(curl);
+		// Free the payload_text after it's been uploaded
+		g_free(payload_text);
+	} else {
+		TRACE(TRACE_ERR, "Unable to init cURL");
+		return false;
+	}
+
+	return (int)res;
+
+}
+
 /* Sends a message. */
 int send_mail(DbmailMessage *message,
 		const char *to, const char *from,
@@ -2307,6 +2555,7 @@ int send_mail(DbmailMessage *message,
 	char *escaped_from = NULL;
 	char *sendmail_command = NULL;
 	Field_T sendmail, postmaster;
+	Field_T smtp_host;
 	int result;
 	char *buf;
 
@@ -2320,6 +2569,22 @@ int send_mail(DbmailMessage *message,
 			from = DEFAULT_POSTMASTER;
 	}
 
+	if (config_get_value("SMTP_HOST", "DBMAIL", smtp_host) == 0){
+		TRACE(TRACE_DEBUG, "Sending mail via cURL, smtp_host [%s]", smtp_host);
+		if (parse_and_escape(to, &escaped_to) < 0) {
+			TRACE(TRACE_NOTICE, "could not prepare 'to' address.");
+			return 1;
+		}
+		if (parse_and_escape(from, &escaped_from) < 0) {
+			g_free(escaped_to);
+			TRACE(TRACE_NOTICE, "could not prepare 'from' address.");
+			return 1;
+		}
+		result = send_smtpmail(message, escaped_to, escaped_from);
+		g_free(escaped_to);
+		g_free(escaped_from);
+		return result;
+	}
 	if (config_get_value("SENDMAIL", "DBMAIL", sendmail) < 0) {
 		TRACE(TRACE_ERR, "error getting value for SENDMAIL in DBMAIL section of dbmail.conf.");
 		return -1;
@@ -2403,7 +2668,7 @@ int send_mail(DbmailMessage *message,
 	if (!sendmail_external)
 		g_free(sendmail_command);
 	return 0;
-} 
+}
 
 static int valid_sender(const char *addr) 
 {
@@ -2511,7 +2776,10 @@ static int send_reply(DbmailMessage *message, const char *body, GList *aliases)
 
 	usubject = dbmail_iconv_decode_text(subject);
        	unewsubject = g_strconcat("Re: ", usubject, NULL);
-	newsubject = g_mime_utils_header_encode_text(NULL, unewsubject, message->charset);
+
+	GMimeFormatOptions * format_options = g_mime_format_options_new ();
+	newsubject = g_mime_utils_header_encode_text(format_options, unewsubject, message->charset);
+	g_mime_format_options_free(format_options);
 	g_free(usubject);
 	g_free(unewsubject);
 
@@ -2640,8 +2908,6 @@ static int execute_auto_ran(DbmailMessage *message, uint64_t useridnr)
 	return 0;
 }
 
-
-
 int send_forward_list(DbmailMessage *message, GList *targets, const char *from)
 {
 	int result = 0;
@@ -2685,7 +2951,6 @@ int send_forward_list(DbmailMessage *message, GList *targets, const char *from)
 			} else if (to[0] == '|') {
 				// The forward is a command to execute.
 				result |= send_mail(message, "", "", NULL, SENDRAW, to+1);
-
 			} else {
 				// The forward is an email address.
 				result |= send_mail(message, to, from, NULL, SENDRAW, SENDMAIL);
@@ -2848,10 +3113,8 @@ int insert_messages(DbmailMessage *message, List_T dsnusers)
 
 		/* Each user may also have a list of external forwarding addresses. */
 		if (g_list_length(delivery->forwards) > 0) {
-
 			TRACE(TRACE_DEBUG, "delivering to external addresses");
 			const char *from = dbmail_message_get_header(message, "Return-Path");
-
 			/* Forward using the temporary stored message. */
 			if (send_forward_list(message, delivery->forwards, from)) {
 				/* If forward fails, tell the sender that we're
@@ -2863,10 +3126,10 @@ int insert_messages(DbmailMessage *message, List_T dsnusers)
 				g_free((char *)from);
 			}
 		}
+
 		if (! p_list_next(dsnusers))
 			break;
 		dsnusers = p_list_next(dsnusers);
-
 	}
 
 	/* Always delete the temporary message, even if the delivery failed.
