@@ -1,7 +1,7 @@
 /*
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -25,6 +25,13 @@
 
 #include "dbmail.h"
 
+/* Loudness and assumptions. */
+int verbose = 0;
+int no_to_all = 0;
+int yes_to_all = 0;
+int reallyquiet = 0;
+int quiet = 0;
+
 extern char configFile[PATH_MAX];
 
 #define SHADOWFILE "/etc/shadow"
@@ -35,12 +42,6 @@ static char csalt[] = "........";
 static char *bgetpwent(const char *filename, const char *name);
 static char *cget_salt(void);
 
-int verbose = 0;
-int no_to_all = 0;
-int yes_to_all = 0;
-int reallyquiet = 0;
-int quiet = 0;
-
 int do_add(const char * const user,
            const char * const password, const char * const enctype,
            const uint64_t maxmail, const uint64_t clientid,
@@ -52,36 +53,48 @@ int do_add(const char * const user,
 	int result;
 
 	if (no_to_all) {
+		TRACE(TRACE_INFO,"Pretending to add user %s with password type %s, %" PRIu64 " bytes mailbox limit and clientid %" PRIu64 "\n",
+			user, enctype, maxmail, clientid);
 		qprintf("Pretending to add user %s with password type %s, %" PRIu64 " bytes mailbox limit and clientid %" PRIu64 "\n",
 			user, enctype, maxmail, clientid);
 		return 1;
 	}
 
+	qprintf("Adding user %s with password type %s,%" PRIu64 " "
+		"bytes mailbox limit and clientid %" PRIu64 "...\n",
+		user, enctype, maxmail, clientid);
 	TRACE(TRACE_DEBUG, "Adding user %s with password type %s,%" PRIu64 " "
-		"bytes mailbox limit and clientid %" PRIu64 "... ", 
+		"bytes mailbox limit and clientid %" PRIu64 "... ",
 		user, enctype, maxmail, clientid);
 
 	if ((result = auth_user_exists(user, &useridnr))) {
-		qerrorf("Failed: user name already exists\n");
+		qprintf("Failed: user name already exists\n");
+		TRACE(TRACE_ERR,"Failed: user name already exists");
 		return result;
 	}
 
 	if (auth_adduser(user, password, enctype, clientid, maxmail, &useridnr) < 0) {
-		qerrorf("Failed: unable to create user\n");
+		qprintf("Failed: unable to create user\n");
+		TRACE(TRACE_ERR,"Failed: unable to create user");
 		return -1;
 	}
 
-	TRACE(TRACE_DEBUG, "Ok, user added id [%" PRIu64 "]\n", useridnr);
+	qprintf("Ok, user added id [%" PRIu64 "]\n", useridnr);
+	TRACE(TRACE_DEBUG, "Ok, user added id [%" PRIu64 "]", useridnr);
 
 	/* Add an INBOX for the user. */
+	TRACE(TRACE_DEBUG, "Adding INBOX for new user... ");
 	qprintf("Adding INBOX for new user... ");
 
 	if (db_createmailbox("INBOX", useridnr, &mailbox_idnr) < 0) {
+		TRACE(TRACE_ERR, "failed... removing user... ");
 		qprintf("failed... removing user... ");
 		if (auth_delete_user(user)) {
-			qprintf("failed also.\n");
+			qprintf("also failed.\n");
+			TRACE(TRACE_ERR, "removing user also failed");
 		} else {
 			qprintf("done.\n");
+			TRACE(TRACE_ERR, "removing user done");
 		}
 		return -1;
 	}
@@ -620,6 +633,7 @@ int do_show(const char * const name)
 
 		/* show all users and their aliases */
 		users = auth_get_known_users();
+		TRACE(TRACE_INFO, "Number of users [%d]", g_list_length(users));
 		if (g_list_length(users) > 0) {
 			users = g_list_first(users);
 			while (users) {
@@ -633,16 +647,16 @@ int do_show(const char * const name)
 		g_list_free(g_list_first(users));
 
 		qprintf("\n-- forwards --\n");
+		TRACE(TRACE_INFO, "-- forwards --");
 
 		/* show all aliases with forwarding addresses */
 		aliases = auth_get_known_aliases();
 		aliases = g_list_dedup(aliases, (GCompareFunc)strcmp, TRUE);
+		TRACE(TRACE_INFO, "Number of aliases [%d]", g_list_length(aliases));
 		if (g_list_length(aliases) > 0) {
 			aliases = g_list_first(aliases);
 			while (aliases) {
 				show_alias(aliases->data, 1);
-				if (! g_list_next(aliases))
-					break;
 				aliases = g_list_next(aliases);
 			}
 			g_list_foreach(aliases,(GFunc)g_free,NULL);
@@ -674,49 +688,49 @@ static int show_alias(const char * const name, int concise)
 	result = auth_check_user_ext(name,&userids,&forwards,0);
 	
 	if (!result) {
-		qerrorf("Nothing found searching for [%s].\n", name);
+		TRACE(TRACE_INFO, "Nothing found searching for [%s]", name);
 		return 1;
 	}
 
 	if (forwards) {
+		GString *fwdlist = g_list_join(forwards,",");
+		TRACE(TRACE_DEBUG, "Forward [%s] to [%s]", name, fwdlist->str);
 		if (concise) {
-			GString *fwdlist = g_list_join(forwards,",");
 			printf("%s: %s\n", name, fwdlist->str);
-			g_string_free(fwdlist, TRUE);
 		} else {
-			GString *fwdlist = g_list_join(forwards,", ");
 			printf("forward [%s] to [%s]\n", name, fwdlist->str);
-			g_string_free(fwdlist, TRUE);
 		}
+		g_string_free(fwdlist, TRUE);
 		g_list_destroy(g_list_first(forwards));
 	}
 	
 	userids = g_list_first(userids);
-	if (userids) {
-		while (userids) {
-			username = auth_get_userid(*(uint64_t *)userids->data);
-			if (!username) {
-				// FIXME: This is a dangling entry. These should be removed
-				// by dbmail-util. This method of identifying dangling aliases
-				// should go into maintenance.c at some point.
-			} else {
-				if (!concise) // Don't print for concise
-				printf("deliver [%s] to [%s]\n", name, username);
-			}
-			g_free(username);
-			if (! g_list_next(userids))
-				break;
-			userids = g_list_next(userids);
+	while (userids) {
+		username = auth_get_userid(*(uint64_t *)userids->data);
+		TRACE(TRACE_DEBUG, "Deliver [%s] to [%s]", name, username);
+		if (!username) {
+			// FIXME: This is a dangling entry. These should be removed
+			// by dbmail-util. This method of identifying dangling aliases
+			// should go into maintenance.c at some point.
+			TRACE(TRACE_WARNING, "Dangling entry [%s]", username);
+		} else {
+			if (!concise) // Don't print for concise
+			printf("deliver [%s] to [%s]\n", name, username);
 		}
-		g_list_free(g_list_first(userids));
+		g_free(username);
+		if (! g_list_next(userids))
+			break;
+		userids = g_list_next(userids);
 	}
+	userids = g_list_first(userids);
+	g_list_free_full(g_steal_pointer (&userids), g_free);
 	return 0;
 }
 
 /* TODO: Non-concise output, like the old dbmail-users used to have. */
 static int show_user(uint64_t useridnr, int concise UNUSED)
 {
-	uint64_t cid, quotum, quotumused;
+	uint64_t cid=0, quotum=0, quotumused=0;
 	GList *userlist = NULL;
 	char *username;
 
@@ -725,7 +739,7 @@ static int show_user(uint64_t useridnr, int concise UNUSED)
 	dm_quota_user_get(useridnr, &quotumused);
         
 	GList *out = NULL;
-	GString *s = g_string_new("");
+	GString *s = NULL;
 	
 	username = auth_get_userid(useridnr);
 	out = g_list_append_printf(out,"%s", username);
@@ -744,6 +758,7 @@ static int show_user(uint64_t useridnr, int concise UNUSED)
 		userlist = g_list_first(userlist);
 		s = g_list_join(userlist,",");
 		g_list_append_printf(out,"%s", s->str);
+		g_string_free(s,TRUE);
 		g_list_foreach(userlist,(GFunc)g_free, NULL);
 	} else {
 		g_list_append_printf(out,"");
@@ -752,6 +767,7 @@ static int show_user(uint64_t useridnr, int concise UNUSED)
 	s = g_list_join(out,":");
 	printf("%s\n", s->str);
 	g_string_free(s,TRUE);
+	g_list_free(g_list_first(out));
 	return 0;
 }
 
@@ -849,7 +865,7 @@ char *bgetpwent(const char *filename, const char *name)
 	return "";
 }
 
-char *cget_salt()
+char *cget_salt(void)
 {
 	unsigned long seed[2];
 	const char *const seedchars =
