@@ -2,7 +2,7 @@
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -28,12 +28,41 @@
  */
 
 #include "dbmail.h"
-#define THIS_MODULE "maintenance"
-#define PNAME "dbmail/maintenance"
+#define THIS_MODULE "util"
+#define PNAME "dbmail/util"
 
 extern DBParam_T db_params;
 #define DBPFX db_params.pfx
 
+/** list of tables used in dbmail, it is a duplicate found in dm_db.c*/
+#define DB_NTABLES 24
+const char *DB_TABLENAMES[DB_NTABLES] = {
+	"acl",
+	"aliases",
+	"authlog",
+	"auto_notifications",
+	"auto_replies",
+	"envelope",
+	"filters",
+	"header",
+	"headername",
+	"headervalue",
+	"keywords",
+	"mailboxes",
+	"messages",
+	"mimeparts",
+	"partlists",
+	"pbsp",
+	"physmessage",
+	"referencesfield",
+	"replycache",
+	"sievescripts",
+	"subscription",
+	"upgrade_steps",
+	"usermap",
+	"users"
+};
+
 /* Loudness and assumptions. */
 int yes_to_all = 0;
 int no_to_all = 0;
@@ -61,7 +90,7 @@ static int do_check_replycache(const char *timespec);
 static int do_vacuum_db(void);
 static int do_rehash(void);
 static int do_migrate(int migrate_limit);
-static int do_upgrade_schema(void);
+static int do_check_empty_envelope(void);
 
 int do_showhelp(void) {
 	printf("*** dbmail-util ***\n");
@@ -71,35 +100,42 @@ int do_showhelp(void) {
 //	0........10........20........30........40........50........60........70........80
 	"Use this program to maintain your DBMail database.\n"
 	"See the man page for more info. Summary:\n\n"
-	"     -a        perform all checks (in this release: -ctubpds)\n"
-	"     -c        clean up database (optimize/vacuum)\n"
-	"     -t        test for message integrity\n"
-	"     -b        body/header/envelope cache check\n"
-	"     -p        purge messages have the DELETE status set\n"
-	"     -d        set DELETE status for deleted messages\n"
-	"     -s        remove dangling/invalid aliases and forwards\n"
-	"     -r time   clear the replycache used for autoreply/vacation\n"
-	"     -l time   clear the IP log used for IMAP/POP-before-SMTP\n"
-	"               the time syntax is [<hours>h][<minutes>m]\n"
-	"               valid examples: 72h, 4h5m, 10m\n"
-	"     -M        migrate legacy 2.2.x messageblks to mimeparts table\n"
-	"     --rehash  Rebuild hash keys for stored messages\n"
-	"     --erase days  Delete messages older than date in INBOX/Trash \n"
-	"     --move  days  Move messages from INBOX to INBOX/Trash\n"
-	"     --inbox name  Inbox folder to move from, used in conjunction with --move\n"
-	"     --trash name  Trash folder to move to, used in conjunction with --move\n"
-	"     -m limit  limit migration to [limit] number of physmessages. Default 10000 per run\n"
+	"     -a, --all-checks         perform all checks\n"
+	"                              (--check-body --set-deleted --purge-deleted\n"
+	"                              --remove-invalid-aliases --test-integrity)\n"
+	"     -c, --clean-database     clean up database (optimize/vacuum)\n"
+	"     -t, --test-integrity     test for message integrity\n"
+	"     -b, --check-body         body/header/envelope cache check\n"
+	"     -e, --check-empty-cache  empty envelope cache check\n"
+	"     -p, --purge-deleted      purge messages have the DELETE status set\n"
+	"     -d, --set-deleted        set DELETE status for deleted messages\n"
+	"     -s, --remove-invalid-aliases\n"
+	"                              remove dangling/invalid aliases and forwards\n"
+	"     -r, --clear-replycache time   clear the replycache used for autoreply/vacation\n"
+	"     -l, --clear-iplog time   clear the IP log used for IMAP/POP-before-SMTP\n"
+	"                              the time syntax is [<hours>h][<minutes>m]\n"
+	"                              valid examples: 72h, 4h5m, 10m\n"
+	"     -M, --migrate-legacy     migrate legacy 2.2.x messageblks to mimeparts table\n"
+	"     -m, --migrate-limit limit\n"
+	"                              limit migration to [limit] number of\n"
+	"                              physmessages. Default 10000 per run\n"
+	"     --rehash                 Rebuild hash keys for stored messages\n"
+	"     --erase days             Delete messages older than date in INBOX/Trash \n"
+	"     --move  days             Move messages from INBOX to INBOX/Trash\n"
+	"     --inbox name             Inbox folder to move from, used in conjunction with --move\n"
+	"     --trash name             Trash folder to move to, used in conjunction with --move\n"
 	"\nCommon options for all DBMail utilities:\n"
-	"     -f file   specify an alternative config file\n"
-	"     -q        quietly skip interactive prompts\n"
-	"               use twice to suppress error messages\n"
-	"     -n        show the intended action but do not perform it, no to all\n"
-	"     -y        perform all proposed actions, as though yes to all\n"
-	"     -v        verbose details\n"
-	"     -V        show the version\n"
-	"     -h        show this help message\n"
-
-	);
+	"     -f, --config file  specify an alternative config file\n"
+	"               Default: %s\n"
+	"     -q, --quiet        quietly skip interactive prompts\n"
+	"                        use twice to suppress error messages\n"
+	"     -n, --no           show the intended action but do not perform it,\n"
+	"                        no to all\n"
+	"     -y, --yes          perform all proposed actions, yes to all\n"
+	"     -v, --verbose      verbose details\n"
+	"     -V, --version      show the version\n"
+	"     -h, --help         show this help message\n"
+	, configFile);
 
 	return 0;
 }
@@ -108,22 +144,39 @@ int main(int argc, char *argv[])
 {
 	int check_integrity = 0;
 	int check_iplog = 0, check_replycache = 0;
+	int check_empty_envelope = 0;
 	char *timespec_iplog = NULL, *timespec_replycache = NULL;
 	int vacuum_db = 0, purge_deleted = 0, set_deleted = 0, dangling_aliases = 0, rehash = 0, move_old = 0, erase_old = 0;
 	int show_help = 0;
 	int do_nothing = 1;
 	int is_header = 0;
-	int upgrade_schema = 0;
 	int migrate = 0, migrate_limit = 10000;
 	static struct option long_options[] = {
-		{ "rehash", 0, 0, 0 },
-		{ "move", 1, 0, 0 },
-		{ "erase", 1, 0, 0 },
-		{ "trash", 1, 0, 0 },
-		{ "inbox", 1, 0, 0 },
-		{ "upgrade-schema", 0, 0, 0 },
-		{ "upgrade", 0, 0, 0 },
-		{ 0, 0, 0, 0 }
+		{"all-checks", no_argument, NULL, 'a'},
+		{"clean-database", no_argument, NULL, 'c'},
+		{"test-integrity", no_argument, NULL, 't'},
+		{"check-body", no_argument, NULL, 'b'},
+		{"check-empty-cache", no_argument, NULL, 'e'},
+		{"purge-deleted", no_argument, NULL, 'p'},
+		{"set-deleted", no_argument, NULL, 'd'},
+		{"remove-invalid-aliases", no_argument, NULL, 's'},
+		{"clear-replycache", required_argument, NULL, 'r'},
+		{"clear-iplog", required_argument, NULL, 't'},
+		{"migrate-legacy", no_argument, NULL, 'M'},
+		{"migrate-limit", required_argument, 0, 'm'},
+		{"rehash", no_argument, NULL, 0},
+		{"move", required_argument, NULL, 0},
+		{"erase", required_argument, NULL, 0},
+		{"trash", required_argument, NULL, 0},
+		{"inbox", required_argument, NULL, 0},
+		{"config",    required_argument, NULL, 'f'},
+		{"quiet",     no_argument, NULL, 'q'},
+		{"no",        no_argument, NULL, 'n'},
+		{"yes",       no_argument, NULL, 'y'},
+		{"help",      no_argument, NULL, 'h'},
+		{"verbose",   no_argument, NULL, 'v'},
+		{"version",   no_argument, NULL, 'V'},
+		{0, 0, 0, 0}
 	};
 	int opt_index = 0;
 	int opt;
@@ -140,7 +193,7 @@ int main(int argc, char *argv[])
 
 	/* get options */
 	opterr = 0;		/* suppress error message from getopt() */
-	while ((opt = getopt_long(argc, argv, "-acgbtl:r:pudsMm:" "i" "f:qnyvVh", long_options, &opt_index)) != -1) {
+	while ((opt = getopt_long(argc, argv, "-abcegtl:r:pudsMm:" "i" "f:qnyvVh", long_options, &opt_index)) != -1) {
 		/* The initial "-" of optstring allows unaccompanied
 		 * options and reports them as the optarg to opt 1 (not '1') */
 		switch (opt) {
@@ -166,10 +219,6 @@ int main(int argc, char *argv[])
 			if (strcmp(long_options[opt_index].name,"inbox")==0) {
 				mbinbox_name = optarg;
 			}
-			
-			if (strcmp(long_options[opt_index].name,"upgrade-schema")==0) {
-				upgrade_schema = 1;
-			}
 			break;
 		case 'a':
 			/* This list should be kept up to date. */
@@ -192,6 +241,11 @@ int main(int argc, char *argv[])
 			do_nothing = 0;
 			break;
 
+		case 'e':
+			check_empty_envelope = 1;
+			do_nothing = 0;
+			break;
+
 		case 'p':
 			purge_deleted = 1;
 			do_nothing = 0;
@@ -240,7 +294,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'i':
-			qerrorf("Interactive console is not supported in this release.\n");
+			qprintf("Interactive console is not supported in this release.\n");
 			return 1;
 
 		/* Common options */
@@ -270,7 +324,7 @@ int main(int argc, char *argv[])
 				memset(configFile, 0, sizeof(configFile));
 				strncpy(configFile, optarg, sizeof(configFile)-1);
 			} else {
-				qerrorf("dbmail-util: -f requires a filename\n\n" );
+				qprintf("dbmail-util: -f requires a filename\n\n" );
 				return 1;
 			}
 			break;
@@ -299,26 +353,30 @@ int main(int argc, char *argv[])
  	/* Don't make any changes unless specifically authorized. */
  	if (!yes_to_all) {
 		qprintf("Choosing dry-run mode. No changes will be made at this time.\n");
+		TRACE(TRACE_INFO, "Choosing dry-run mode. No changes will be made at this time.");
 		no_to_all = 1;
  	}
 
 	config_read(configFile);
-	SetTraceLevel("DBMAIL");
+	SetTraceLevel("UTIL");
 	GetDBParams();
 
 	qverbosef("Opening connection to database... \n");
 	if (db_connect() != 0) {
-		qerrorf("Failed. An error occured. Please check log.\n");
+		qprintf("Failed. An error occured. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
 		return -1;
 	}
 
 	qverbosef("Opening connection to authentication... \n");
 	if (auth_connect() != 0) {
-		qerrorf("Failed. An error occured. Please check log.\n");
+		qprintf("Failed. An error occured. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
 		return -1;
 	}
 
 	qverbosef("Ok. Connected.\n");
+	TRACE(TRACE_INFO, "Ok. Connected.");
 
 	if (erase_old) do_erase_old(days_erase, mbtrash_name);
 	if (move_old) do_move_old(days_move, mbinbox_name, mbtrash_name);
@@ -332,28 +390,38 @@ int main(int argc, char *argv[])
 	if (vacuum_db) do_vacuum_db();
 	if (rehash) do_rehash();
 	if (migrate) do_migrate(migrate_limit);
-	if (upgrade_schema) do_upgrade_schema();
+	if (check_empty_envelope) do_check_empty_envelope();
 
 	if (!has_errors && !serious_errors) {
 		qprintf("\nMaintenance done. No errors found.\n");
+		TRACE(TRACE_INFO, "Maintenance done. No errors found.");
 	} else {
-		qerrorf("\nMaintenance done. Errors were found");
+		qprintf("\nMaintenance done. Errors were found");
+		TRACE(TRACE_INFO, "Maintenance done. Errors were found");
 		if (serious_errors) {
-			qerrorf(" but not fixed due to failures.\n");
-			qerrorf("Please check the logs for further details, "
+			qprintf(" but not fixed due to failures.\n");
+			qprintf("Please check the logs for further details, "
 				"turning up the trace level as needed.\n");
+			TRACE(TRACE_INFO, "Errors not fixed due to failures.");
 			// Indicate that something went really wrong
 			has_errors = 3;
 		} else if (no_to_all) {
-			qerrorf(" but not fixed.\n");
-			qerrorf("Run again with the '-y' option to "
+			qprintf(" but not fixed.\n");
+			qprintf("Run again with the '-y' option to "
 				"repair the errors.\n");
+			TRACE(TRACE_INFO, "Errors found but not fixed");
+			TRACE(TRACE_INFO,
+				"Run again with the '-y' option to repair the errors."
+			);
 			// Indicate that the program should be run with -y
 			has_errors = 2;
 		} else if (yes_to_all) {
-			qerrorf(" and fixed.\n");
-			qerrorf("We suggest running dbmail-util again to "
+			qprintf(" and fixed.\n");
+			qprintf("Recommend running dbmail-util again to "
 				"confirm that all errors were repaired.\n");
+			TRACE(TRACE_INFO, "Errors found and fixed");
+			TRACE(TRACE_INFO, "Recommend running dbmail-util again to "
+				"confirm that all errors were repaired.");
 			// Indicate that the program should be run again
 			has_errors = 1;
 		}
@@ -480,28 +548,33 @@ static int db_deleted_count(uint64_t * rows)
 	return t;
 }
 
-
 int do_purge_deleted(void)
 {
 	uint64_t deleted_messages;
 
 	if (no_to_all) {
 		qprintf("\nCounting messages with DELETE status...\n");
+		TRACE(TRACE_INFO, "Counting messages with DELETE status...");
 		if (! db_deleted_count(&deleted_messages)) {
-			qerrorf ("Failed. An error occured. Please check log.\n");
+			qprintf ("Failed. An error occured. Please check log.\n");
+			TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf("Ok. [%" PRIu64 "] messages have DELETE status.\n", deleted_messages);
+		TRACE(TRACE_INFO, "Ok. [%" PRIu64 "] messages have DELETE status.", deleted_messages);
 	}
 	if (yes_to_all) {
 		qprintf("\nDeleting messages with DELETE status...\n");
+		TRACE(TRACE_INFO, "Deleting messages with DELETE status...");
 		if (! db_deleted_purge()) {
-			qerrorf ("Failed. An error occured. Please check log.\n");
+			qprintf ("Failed. An error occured. Please check log.\n");
+			TRACE(TRACE_INFO, "Failed. An error occured. Please check log");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf("Ok. Messages deleted.\n");
+		TRACE(TRACE_INFO, "Ok. Messages deleted.");
 	}
 	return 0;
 }
@@ -513,28 +586,36 @@ int do_set_deleted(void)
 	if (no_to_all) {
 		// TODO: Count messages to delete.
 		qprintf("\nCounting deleted messages that need the DELETE status set...\n");
+		TRACE(TRACE_INFO, "Counting deleted messages that need the DELETE status set...");
 		if (! db_count_deleted(&messages_set_to_delete)) {
-			qerrorf ("Failed. An error occured. Please check log.\n");
+			qprintf ("Failed. An error occured. Please check log.\n");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf("Ok. [%" PRIu64 "] messages need to be set for deletion.\n", messages_set_to_delete);
+		TRACE(TRACE_INFO, "Ok. [%" PRIu64 "] messages need to be set for deletion.", messages_set_to_delete);
 	}
 	if (yes_to_all) {
 		qprintf("\nSetting DELETE status for deleted messages...\n");
+		TRACE(TRACE_INFO, "Setting DELETE status for deleted messages...");
 		if (! db_set_deleted()) {
-			qerrorf ("Failed. An error occured. Please check log.\n");
+			qprintf ("Failed. An error occured. Please check log.\n");
+			TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf("Ok. Messages set for deletion.\n");
-		qprintf("Re-calculating used quota for all users...\n");
+		TRACE(TRACE_INFO, "Ok. Messages set for deletion.");
+		qprintf("\nRe-calculating used quota for all users...\n");
+		TRACE(TRACE_INFO, "Re-calculating used quota for all users...");
 		if (dm_quota_rebuild() < 0) {
-			qerrorf ("Failed. An error occured. Please check log.\n");
+			qprintf ("Failed. An error occured. Please check log.\n");
+			TRACE(TRACE_INFO, "Failed. An error occured. Please check log");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf("Ok. Used quota updated for all users.\n");
+		TRACE(TRACE_INFO, "Ok. Used quota updated for all users.");
 	}
 	return 0;
 }
@@ -553,7 +634,8 @@ GList *find_dangling_aliases(const char * const name)
 	result = auth_check_user_ext(name,&uids,&fwds,0);
 	
 	if (!result) {
-		qerrorf("Nothing found searching for [%s].\n", name);
+		qprintf("Nothing found searching for [%s].\n", name);
+		TRACE(TRACE_INFO, "Nothing found searching for [%s].", name);
 		serious_errors = 1;
 		return dangling;
 	}
@@ -579,10 +661,14 @@ int do_dangling_aliases(void)
 	int result = 0;
 	GList *aliases = NULL;
 
-	if (no_to_all)
+	if (no_to_all) {
 		qprintf("\nCounting aliases with nonexistent delivery userid's...\n");
-	if (yes_to_all)
+		TRACE(TRACE_INFO, "nCounting aliases with nonexistent delivery userid's...");
+	}
+	if (yes_to_all) {
 		qprintf("\nRemoving aliases with nonexistent delivery userid's...\n");
+		TRACE(TRACE_INFO, "Removing aliases with nonexistent delivery userid's...");
+	}
 
 	aliases = auth_get_known_aliases();
 	aliases = g_list_dedup(aliases, (GCompareFunc)strcmp, TRUE);
@@ -597,9 +683,13 @@ int do_dangling_aliases(void)
 			g_snprintf(deliver_to, 21, "%" PRIu64 "", *(uint64_t *)dangling->data);
 			qverbosef("Dangling alias [%s] delivers to nonexistent user [%s]\n",
 				(char *)aliases->data, deliver_to);
+			TRACE(TRACE_INFO, "Dangling alias [%s] delivers to nonexistent user [%s]",
+				(char *)aliases->data, deliver_to);
 			if (yes_to_all) {
 				if (auth_removealias_ext(aliases->data, deliver_to) < 0) {
-					qerrorf("Error: could not remove alias [%s] deliver to [%s] \n",
+					qprintf("Error: could not remove alias [%s] deliver to [%s]\n",
+						(char *)aliases->data, deliver_to);
+					TRACE(TRACE_INFO, "Error: could not remove alias [%s] deliver to [%s]",
 						(char *)aliases->data, deliver_to);
 					serious_errors = 1;
 					result = -1;
@@ -617,11 +707,10 @@ int do_dangling_aliases(void)
 	}
 	g_list_destroy(g_list_first(aliases));
 
+	qprintf("Ok. Found [%d] dangling aliases.\n", count);
+	TRACE(TRACE_INFO, "Ok. Found [%d] dangling aliases.", count);
 	if (count > 0) {
-		qerrorf("Ok. Found [%d] dangling aliases.\n", count);
 		has_errors = 1;
-	} else {
-		qprintf("Ok. Found [%d] dangling aliases.\n", count);
 	}
 
 	return result;
@@ -644,6 +733,7 @@ int do_check_integrity(void)
 	}
 
 	qprintf("\n%s DBMAIL message integrity...\n", action);
+	TRACE(TRACE_INFO, "%s DBMAIL message integrity...", action);
 
 	/* This is what we do:
 	 3. Check for loose physmessages
@@ -656,18 +746,18 @@ int do_check_integrity(void)
 	/* part 3 */
 	time(&start);
 	qprintf("\n%s DBMAIL physmessage integrity...\n", action);
+	TRACE(TRACE_INFO, "%s DBMAIL physmessage integrity...", action);
 	if ((count = db_icheck_physmessages(cleanup)) < 0) {
-		qerrorf("Failed. An error occurred. Please check log.\n");
+		qprintf("Failed. An error occurred. Please check log.\n");
 		serious_errors = 1;
 		return -1;
 	}
+	qprintf("Ok. Found [%ld] unconnected physmessages.\n", count);
+	TRACE(TRACE_INFO, "Ok. Found [%ld] unconnected physmessages.", count);
 	if (count > 0) {
-		qerrorf("Ok. Found [%ld] unconnected physmessages.\n", count);
 		if (cleanup) {
-			qerrorf("Ok. Orphaned physmessages deleted.\n");
+			qprintf("Ok. Orphaned physmessages deleted.\n");
 		}
-	} else {
-		qprintf("Ok. Found [%ld] unconnected physmessages.\n", count);
 	}
 
 	time(&stop);
@@ -678,45 +768,52 @@ int do_check_integrity(void)
 	/* part 4 */
 	start = stop;
 	qprintf("\n%s DBMAIL partlists integrity...\n", action);
+	TRACE(TRACE_INFO, "%s DBMAIL partlists integrity...", action);
 	if ((count = db_icheck_partlists(cleanup)) < 0) {
-		qerrorf("Failed. An error occurred. Please check log.\n");
+		qprintf("Failed. An error occurred. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occurred. Please check log.");
 		serious_errors = 1;
 		return -1;
 	}
+	qprintf("Ok. Found [%ld] unconnected partlists.\n", count);
+	TRACE(TRACE_INFO, "Ok. Found [%ld] unconnected partlists.", count);
 	if (count > 0) {
-		qerrorf("Ok. Found [%ld] unconnected partlists.\n", count);
 		if (cleanup) {
-			qerrorf("Ok. Orphaned partlists deleted.\n");
+			qprintf("Ok. Orphaned partlists deleted.\n");
+			TRACE(TRACE_INFO, "Ok. Orphaned partlists deleted.");
 		}
-	} else {
-		qprintf("Ok. Found [%ld] unconnected partlists.\n", count);
 	}
 
 	time(&stop);
 	qverbosef("--- %s unconnected partlists took %g seconds\n",
 		action, difftime(stop, start));
+	TRACE(TRACE_INFO, "--- %s unconnected partlists took %g seconds",
+		action, difftime(stop, start));
 	/* end part 4 */
 
 	/*  part 5 */
 	start = stop;
 	qprintf("\n%s DBMAIL mimeparts integrity...\n", action);
+	TRACE(TRACE_INFO, "%s DBMAIL mimeparts integrity...", action);
 	if ((count = db_icheck_mimeparts(cleanup)) < 0) {
-		qerrorf("Failed. An error occurred. Please check log.\n");
+		qprintf("Failed. An error occurred. Please check log.\n");
 		serious_errors = 1;
 		return -1;
 	}
+	TRACE(TRACE_INFO, "Ok. Found [%ld] unconnected mimeparts.", count);
+	qprintf("Ok. Found [%ld] unconnected mimeparts.\n", count);
 	if (count > 0) {
-		qerrorf("Ok. Found [%ld] unconnected mimeparts.\n", count);
 		if (cleanup) {
-			qerrorf("Ok. Orphaned mimeparts deleted.\n");
+			qprintf("Ok. Orphaned mimeparts deleted.\n");
+			TRACE(TRACE_INFO, "Ok. Orphaned mimeparts deleted.");
 		}
-	} else {
-		qprintf("Ok. Found [%ld] unconnected mimeparts.\n", count);
 	}
 
 	time(&stop);
 	qverbosef("--- %s unconnected mimeparts took %g seconds\n",
 		action, difftime(stop, start));
+	TRACE(TRACE_INFO, "--- %s unconnected mimeparts took %g seconds\n",
+		action, difftime(stop, start));
 	/* end part 5 */
 
 	/* part 6 */
@@ -729,43 +826,54 @@ int do_check_integrity(void)
 		}
 	}
 
-        if (! cache_readonly) {
+	if (! cache_readonly) {
 		start = stop;
 		qprintf("\n%s DBMAIL headernames integrity...\n", action);
+		TRACE(TRACE_INFO, "%s DBMAIL headernames integrity...", action);
 		if ((count = db_icheck_headernames(cleanup)) < 0) {
-			qerrorf("Failed. An error occurred. Please check log.\n");
+			qprintf("Failed. An error occurred. Please check log.\n");
+			TRACE(TRACE_INFO, "Failed. An error occurred. Please check log.");
 			serious_errors = 1;
 			return -1;
 		}
 
 		qprintf("Ok. Found [%ld] unconnected headernames.\n", count);
+		TRACE(TRACE_INFO, "Ok. Found [%ld] unconnected headernames.", count);
 		if (count > 0 && cleanup) {
-			qerrorf("Ok. Orphaned headernames deleted.\n");
+			qprintf("Ok. Orphaned headernames deleted.\n");
 		}
 
 		time(&stop);
 		qverbosef("--- %s unconnected headernames took %g seconds\n",
 			action, difftime(stop, start));
+		TRACE(TRACE_INFO, "--- %s unconnected headernames took %g seconds\n",
+			action, difftime(stop, start));
 	}
 	/* end part 6 */
 
 	/* part 7 */
 	start = stop;
 	qprintf("\n%s DBMAIL headervalues integrity...\n", action);
+	TRACE(TRACE_INFO, "%s DBMAIL headervalues integrity...", action);
 	if ((count = db_icheck_headervalues(cleanup)) < 0) {
-		qerrorf("Failed. An error occurred. Please check log.\n");
+		qprintf("Failed. An error occurred. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occurred. Please check log.");
 		serious_errors = 1;
 		return -1;
 	}
 
 	qprintf("Ok. Found [%ld] unconnected headervalues.\n", count);
+	TRACE(TRACE_INFO, "Ok. Found [%ld] unconnected headervalues.\n", count);
 	if (count > 0 && cleanup) {
-		qerrorf("Ok. Orphaned headervalues deleted.\n");
+		qprintf("Ok. Orphaned headervalues deleted.\n");
+		TRACE(TRACE_INFO, "Ok. Orphaned headervalues deleted.");
 	}
 
 	time(&stop);
 	qverbosef("--- %s unconnected headervalues took %g seconds\n",
 		action, difftime(stop, start));
+		TRACE(TRACE_INFO, "--- %s unconnected headervalues took %g seconds\n",
+		action, difftime(stop, start));
 	/* end part 7 */
 
 	g_list_destroy(lost);
@@ -773,6 +881,7 @@ int do_check_integrity(void)
 
 	time(&stop);
 	qverbosef("--- %s block integrity took %g seconds\n", action, difftime(stop, start));
+	TRACE(TRACE_INFO, "--- %s block integrity took %g seconds\n", action, difftime(stop, start));
 
 	return 0;
 }
@@ -784,28 +893,32 @@ static int do_rfc_size(void)
 
 	if (no_to_all) {
 		qprintf("\nChecking DBMAIL for rfcsize field...\n");
+		TRACE(TRACE_INFO, "Checking DBMAIL for rfcsize field...");
 	}
 	if (yes_to_all) {
 		qprintf("\nRepairing DBMAIL for rfcsize field...\n");
+		TRACE(TRACE_INFO, "Repairing DBMAIL for rfcsize field...");
 	}
 	time(&start);
 
 	if (db_icheck_rfcsize(&lost) < 0) {
-		qerrorf("Failed. An error occured. Please check log.\n");
+		qprintf("Failed. An error occured. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
 		serious_errors = 1;
 		return -1;
 	}
 
+	TRACE(TRACE_INFO, "Ok. Found [%d] missing rfcsize values.", g_list_length(lost));
+	qprintf("Ok. Found [%d] missing rfcsize values.\n", g_list_length(lost));
+	TRACE(TRACE_INFO, "Ok. Found [%d] missing rfcsize values.\n", g_list_length(lost));
 	if (g_list_length(lost) > 0) {
-		qerrorf("Ok. Found [%d] missing rfcsize values.\n", g_list_length(lost));
 		has_errors = 1;
-	} else {
-		qprintf("Ok. Found [%d] missing rfcsize values.\n", g_list_length(lost));
 	}
 
 	if (yes_to_all) {
 		if (db_update_rfcsize(lost) < 0) {
-			qerrorf("Error setting the rfcsize values");
+			TRACE(TRACE_INFO, "Error setting the rfcsize values");
+			qprintf("Error setting the rfcsize values");
 			has_errors = 1;
 		}
 	}
@@ -815,7 +928,9 @@ static int do_rfc_size(void)
 	time(&stop);
 	qverbosef("--- checking rfcsize field took %g seconds\n",
 	       difftime(stop, start));
-	
+	TRACE(TRACE_INFO, "--- checking rfcsize field took %g seconds\n",
+	       difftime(stop, start));
+
 	return 0;
 
 }
@@ -827,28 +942,31 @@ static int do_envelope(void)
 
 	if (no_to_all) {
 		qprintf("\nChecking DBMAIL for cached envelopes...\n");
+		TRACE(TRACE_INFO, "Checking DBMAIL for cached envelopes...");
 	}
 	if (yes_to_all) {
 		qprintf("\nRepairing DBMAIL for cached envelopes...\n");
+		TRACE(TRACE_INFO, "Repairing DBMAIL for cached envelopes...");
 	}
 	time(&start);
 
 	if (db_icheck_envelope(&lost) < 0) {
-		qerrorf("Failed. An error occured. Please check log.\n");
+		qprintf("Failed. An error occured. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
 		serious_errors = 1;
 		return -1;
 	}
 
+	TRACE(TRACE_INFO, "Ok. Found [%d] missing envelope values.", g_list_length(lost));
+	qprintf("Ok. Found [%d] missing envelope values.\n", g_list_length(lost));
 	if (g_list_length(lost) > 0) {
-		qerrorf("Ok. Found [%d] missing envelope values.\n", g_list_length(lost));
 		has_errors = 1;
-	} else {
-		qprintf("Ok. Found [%d] missing envelope values.\n", g_list_length(lost));
 	}
 
 	if (yes_to_all) {
 		if (db_set_envelope(lost) < 0) {
-			qerrorf("Error setting the envelope cache");
+			qprintf("Error setting the envelope cache");
+			TRACE(TRACE_INFO, "Error setting the envelope cache");
 			has_errors = 1;
 		}
 	}
@@ -858,7 +976,57 @@ static int do_envelope(void)
 	time(&stop);
 	qverbosef("--- checking envelope cache took %g seconds\n",
 	       difftime(stop, start));
-	
+	TRACE(TRACE_INFO, "--- checking envelope cache took %g seconds\n",
+	       difftime(stop, start));
+
+	return 0;
+
+}
+
+static int do_check_empty_envelope(void)
+{
+	time_t start, stop;
+	GList *lost = NULL;
+
+	if (no_to_all) {
+		qprintf("\nChecking DBMAIL for empty cached envelopes...\n");
+		TRACE(TRACE_INFO, "Checking DBMAIL for empty cached envelopes...");
+	}
+	if (yes_to_all) {
+		qprintf("\nRepairing DBMAIL for empty cached envelopes...\n");
+		TRACE(TRACE_INFO, "Repairing DBMAIL for empty cached envelopes...");
+	}
+	time(&start);
+
+	if (db_icheck_empty_envelope(&lost) < 0) {
+		qprintf("Failed. An error occured. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
+		serious_errors = 1;
+		return -1;
+	}
+
+	qprintf("Ok. Found [%d] empty envelope values.\n", g_list_length(lost));
+	TRACE(TRACE_INFO, "Ok. Found [%d] empty envelope values.", g_list_length(lost));
+	if (g_list_length(lost) > 0) {
+		has_errors = 1;
+	}
+
+	if (yes_to_all) {
+		if (db_set_envelope(lost) < 0) {
+			qprintf("Error setting the envelope cache");
+			TRACE(TRACE_INFO, "Error setting the envelope cache");
+			has_errors = 1;
+		}
+	}
+
+	g_list_destroy(lost);
+
+	time(&stop);
+	qverbosef("--- checking empty envelope cache took %g seconds\n",
+	       difftime(stop, start));
+	TRACE(TRACE_INFO, "--- checking empty envelope cache took %g seconds\n",
+	       difftime(stop, start));
+
 	return 0;
 
 }
@@ -878,29 +1046,35 @@ int do_header_cache(void)
 		return -1;
 	}
 	
-	if (no_to_all) 
+	if (no_to_all) {
 		qprintf("\nChecking DBMAIL for cached header values...\n");
-	if (yes_to_all) 
+		TRACE(TRACE_INFO, "Checking DBMAIL for cached header values...");
+	}
+	if (yes_to_all) {
 		qprintf("\nRepairing DBMAIL for cached header values...\n");
-	
+		TRACE(TRACE_INFO, "Repairing DBMAIL for cached header values...");
+	}
+
 	time(&start);
 
 	if (db_icheck_headercache(&lost) < 0) {
-		qerrorf("Failed. An error occured. Please check log.\n");
+		qprintf("Failed. An error occured. Please check log.\n");
+		TRACE(TRACE_INFO, "Failed. An error occured. Please check log.");
 		serious_errors = 1;
 		return -1;
 	}
 
+	TRACE(TRACE_INFO, "Ok. Found [%d] un-cached physmessages.", g_list_length(lost));
+	qprintf("Ok. Found [%d] un-cached physmessages.\n", g_list_length(lost));
+	TRACE(TRACE_INFO, "Ok. Found [%d] un-cached physmessages.\n", g_list_length(lost));
 	if (g_list_length(lost) > 0) {
-		qerrorf("Ok. Found [%d] un-cached physmessages.\n", g_list_length(lost));
 		has_errors = 1;
-	} else {
-		qprintf("Ok. Found [%d] un-cached physmessages.\n", g_list_length(lost));
 	}
 
 	if (yes_to_all) {
 		if (db_set_headercache(lost) < 0) {
-			qerrorf("Error caching the header values ");
+			qprintf("Error caching the header values");
+			TRACE(TRACE_INFO, "Error caching the header values");
 			serious_errors = 1;
 		}
 	}
@@ -910,6 +1084,8 @@ int do_header_cache(void)
 	time(&stop);
 	qverbosef("--- checking cached headervalues took %g seconds\n",
 	       difftime(stop, start));
+	TRACE(TRACE_INFO, "--- checking cached headervalues took %g seconds\n",
+	       difftime(stop, start));
 
 	return 0;
 }
@@ -921,7 +1097,9 @@ int do_check_iplog(const char *timespec)
 	TimeString_T timestring;
 
 	if (find_time(timespec, &timestring) != 0) {
-		qerrorf("\nFailed to find a timestring: [%s] is not <hours>h<minutes>m.\n",
+		qprintf("\nFailed to find a timestring: [%s] is not <hours>h<minutes>m.\n",
+		       timespec);
+		TRACE(TRACE_INFO, "Failed to find a timestring: [%s] is not <hours>h<minutes>m.",
 		       timespec);
 		serious_errors = 1;
 		return -1;
@@ -929,24 +1107,31 @@ int do_check_iplog(const char *timespec)
 
 	if (no_to_all) {
 		qprintf("\nCounting IP entries older than [%s]...\n", timestring);
+		TRACE(TRACE_INFO, "Counting IP entries older than [%s]...", timestring);
 		if (db_count_iplog(timestring, &log_count) < 0) {
-			qerrorf("Failed. An error occured. Check the log.\n");
+			qprintf("Failed. An error occured. Check the log.\n");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf("Ok. [%" PRIu64 "] IP entries are older than [%s].\n",
 		    log_count, timestring);
+		TRACE(TRACE_INFO, "Ok. [%" PRIu64 "] IP entries are older than [%s].",
+		    log_count, timestring);
 	}
 	if (yes_to_all) {
 		qprintf("\nRemoving IP entries older than [%s]...\n", timestring);
+		TRACE(TRACE_INFO, "Removing IP entries older than [%s]...", timestring);
 		if (! db_cleanup_iplog(timestring)) {
-			qerrorf("Failed. Please check the log.\n");
+			qprintf("Failed. Please check the log.\n");
+			TRACE(TRACE_INFO, "Failed. Please check the log.");
 			serious_errors = 1;
 			return -1;
 		}
 
 		qprintf("Ok. IP entries older than [%s] removed.\n",
 		       timestring);
+		TRACE(TRACE_INFO, "Ok. IP entries older than [%s] removed.\n",
+		       timestring);
 	}
 	return 0;
 }
@@ -957,7 +1142,9 @@ int do_check_replycache(const char *timespec)
 	TimeString_T timestring;
 
 	if (find_time(timespec, &timestring) != 0) {
-		qerrorf("\nFailed to find a timestring: [%s] is not <hours>h<minutes>m.\n",
+		qprintf("\nFailed to find a timestring: [%s] is not <hours>h<minutes>m.\n",
+		       timespec);
+		TRACE(TRACE_INFO, "Failed to find a timestring: [%s] is not <hours>h<minutes>m.",
 		       timespec);
 		serious_errors = 1;
 		return -1;
@@ -965,23 +1152,30 @@ int do_check_replycache(const char *timespec)
 
 	if (no_to_all) {
 		qprintf("\nCounting RC entries older than [%s]...\n", timestring);
+		TRACE(TRACE_INFO, "Counting RC entries older than [%s]...", timestring);
 		if (db_count_replycache(timestring, &log_count) < 0) {
-			qerrorf("Failed. An error occured. Check the log.\n");
+			qprintf("Failed. An error occured. Check the log.\n");
+			TRACE(TRACE_INFO, "Failed. An error occured. Check the log.");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf("Ok. [%" PRIu64 "] RC entries are older than [%s].\n",
 		    log_count, timestring);
+		TRACE(TRACE_INFO, "Ok. [%" PRIu64 "] RC entries are older than [%s].\n",
+		    log_count, timestring);
 	}
 	if (yes_to_all) {
 		qprintf("\nRemoving RC entries older than [%s]...\n", timestring);
+		TRACE(TRACE_INFO, "Removing RC entries older than [%s]...", timestring);
 		if (! db_cleanup_replycache(timestring)) {
-			qerrorf("Failed. Please check the log.\n");
+			qprintf("Failed. Please check the log.\n");
+			TRACE(TRACE_INFO, "Failed. Please check the log.");
 			serious_errors = 1;
 			return -1;
 		}
 
 		qprintf("Ok. RC entries were older than [%s] cleaned.\n", timestring);
+		TRACE(TRACE_INFO, "Ok. RC entries were older than [%s] cleaned.\n", timestring);
 	}
 	return 0;
 }
@@ -990,17 +1184,21 @@ int do_vacuum_db(void)
 {
 	if (no_to_all) {
 		qprintf("\nVacuum and optimize not performed.\n");
+		TRACE(TRACE_INFO, "Vacuum and optimize not performed.");
 	}
 	if (yes_to_all) {
 		qprintf("\nVacuuming and optimizing database...\n");
+		TRACE(TRACE_INFO, "Vacuuming and optimizing database...");
 		fflush(stdout);
 		if (db_cleanup() < 0) {
-			qerrorf("Failed. Please check the log.\n");
+			qprintf("Failed. Please check the log.\n");
+			TRACE(TRACE_INFO, "Failed. Please check the log.");
 			serious_errors = 1;
 			return -1;
 		}
 
 		qprintf("Ok. Database cleaned up.\n");
+		TRACE(TRACE_INFO, "Ok. Database cleaned up.");
 	}
 	return 0;
 }
@@ -1009,12 +1207,14 @@ int do_rehash(void)
 {
 	if (yes_to_all) {
 		qprintf ("Rebuild hash keys for stored message chunks...\n");
+		TRACE(TRACE_INFO, "Rebuild hash keys for stored message chunks...");
 		if (db_rehash_store() == DM_EQUERY) {
-			qerrorf("Failed. Please check the log.\n");
+			qprintf("Failed. Please check the log.\n");
 			serious_errors = 1;
 			return -1;
 		}
 		qprintf ("Ok. Hash keys rebuild successfully.\n");
+		TRACE(TRACE_INFO, "Ok. Hash keys rebuild successfully.");
 	}
 
 	return 0;
@@ -1029,11 +1229,35 @@ int do_migrate(int migrate_limit)
 	DbmailMessage *m;
 	
 	qprintf ("Migrate legacy 2.2.x messageblks to mimeparts...\n");
+	TRACE(TRACE_INFO, "Migrate legacy 2.2.x messageblks to mimeparts...");
 	if (!yes_to_all) {
+		c = db_con_get();
+		TRY
+			db_begin_transaction(c);
+			r = db_query(c, "SELECT count(physmessage_id) FROM %smessageblks", DBPFX);
+			while (db_result_next(r))
+			{
+				count = db_result_get_u64(r,0);
+			}
+			db_commit_transaction(c);
+		CATCH(SQLException)
+			LOG_SQLERROR;
+			db_rollback_transaction(c);
+			return -1;
+		FINALLY
+			db_con_close(c);
+		END_TRY;
+
+		qprintf ("There are %d messageblks available for migration.\n", count);
+		TRACE(TRACE_INFO, "There are %d messageblks available for migration.", count);
+
 		qprintf ("\tmigration skipped. Use -y option to perform migration.\n");
+		TRACE(TRACE_INFO, "  migration skipped. Use -y option to perform migration.");
+
 		return 0;
 	}
 	qprintf ("Preparing to migrate up to %d physmessages.\n", migrate_limit);
+	TRACE(TRACE_INFO, "Preparing to migrate up to %d physmessages.", migrate_limit);
 
 	c = db_con_get();
 	TRY
@@ -1061,39 +1285,17 @@ int do_migrate(int migrate_limit)
 	END_TRY;
 	
 	qprintf ("Migration complete. Migrated %d physmessages.\n", count);
-	return 0;
-}
+	TRACE(TRACE_INFO, "Migration complete. Migrated %d physmessages.", count);
 
-int do_upgrade_schema(void)
-{
-	const char *query = NULL;
-	Connection_T c;	
-	if (!yes_to_all) {
-		qprintf ("\tupgrading skipped. Use -y option to perform migration.\n");
-		return 0;
-	}
-	qprintf ("Preparing to upgrade \n");
 	c = db_con_get();
 	TRY
 		db_begin_transaction(c);
-		
-		switch(db_params.db_driver) {
-			case DM_DRIVER_SQLITE:
-				query = DM_SQLITE_UPGRADE;
-			break;
-			case DM_DRIVER_MYSQL:
-				query = DM_MYSQL_UPGRADE;
-			break;
-			case DM_DRIVER_POSTGRESQL:
-				query = DM_PGSQL_UPGRADE;
-			break;
-			case DM_DRIVER_ORACLE:
-				qprintf ("\tPlease upgrade Oracle manually.\n");
-				return -1;
-			break;
+		r = db_query(c, "SELECT count(physmessage_id) FROM %smessageblks", DBPFX);
+		while (db_result_next(r))
+		{
+			count = db_result_get_u64(r,0);
 		}
-		qprintf ("Executing\n%s\n...",query);
-		db_exec(c, query);
+		db_commit_transaction(c);
 	CATCH(SQLException)
 		LOG_SQLERROR;
 		db_rollback_transaction(c);
@@ -1101,10 +1303,12 @@ int do_upgrade_schema(void)
 	FINALLY
 		db_con_close(c);
 	END_TRY;
-	
-	qprintf ("Upgrading complete.\n");
+
+	qprintf ("There are %d messageblks to be migrated.\n", count);
+	TRACE(TRACE_INFO, "There are %d messageblks to be migrated.", count);
 	return 0;
 }
+
 /* Makes a date/time string: YYYY-MM-DD HH:mm:ss
  * based on current time minus timespec
  * timespec contains: <n>h<m>m for a timespan of n hours, m minutes
@@ -1282,7 +1486,8 @@ int do_move_old (int days, char * mbinbox_name, char * mbtrash_name)
 				db_mailbox_seq_update(mailbox_from, 0);
 			}
 			else {
-				qprintf("User(%" PRIu64 ") doesn't has mailbox(%s)\n", user_id, mbtrash_name);
+				qprintf("User(%" PRIu64 ") doesn't have mailbox(%s)\n", user_id, mbtrash_name);
+				TRACE(TRACE_INFO, "User(%" PRIu64 ") doesn't have mailbox(%s)", user_id, mbtrash_name);
 			}
 		}
 
