@@ -1,8 +1,9 @@
+
 /*
  Copyright (c) 2002 Aaron Stone, aaron@serendipity.cx
  Copyright (c) 2004-2010 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -222,14 +223,15 @@ static int authldap_connect(void)
 			version = LDAP_VERSION3;
 			if (strlen(_ldap_cfg.uri)) {
 				TRACE(TRACE_DEBUG, "connecting to ldap server on [%s] version [%d]", _ldap_cfg.uri, _ldap_cfg.version_int);
-				if ((ret = ldap_initialize(&_ldap_conn, _ldap_cfg.uri) != LDAP_SUCCESS)) 
-					TRACE(TRACE_WARNING, "ldap_initialize() failed %d", ret);
+				if ((ret = ldap_initialize(&_ldap_conn, _ldap_cfg.uri) != LDAP_SUCCESS)) {
+					TRACE(TRACE_EMERG, "ldap_initialize() failed %s", strerror(ret));
+				}
 			} else {
 				char *uri = g_strdup_printf("ldap://%s:%d", _ldap_cfg.hostname, _ldap_cfg.port_int);
 				TRACE(TRACE_DEBUG, "connecting to ldap server on [%s] version [%d]", uri, _ldap_cfg.version_int);
-				if ((ret = ldap_initialize(&_ldap_conn, uri)) != LDAP_SUCCESS) 
-					TRACE(TRACE_EMERG, "ldap_initialize() failed [%d]", ret);
-
+				if ((ret = ldap_initialize(&_ldap_conn, uri)) != LDAP_SUCCESS) {
+					TRACE(TRACE_EMERG, "ldap_initialize() failed [%s]", strerror(ret));
+				}
 				g_free(uri);
 			}
 			break;
@@ -273,7 +275,7 @@ static LDAPMessage * authldap_search(const gchar *query)
 	char **_ldap_attrs = NULL;
 	int err = -1; // Start wanting success
 	int c = 0;
-	const char *err_msg = NULL;
+	char *err_msg = NULL;
 	int c_tries = _ldap_cfg.query_timeout_int;
 	LDAP *_ldap_conn;
 
@@ -293,10 +295,21 @@ static LDAPMessage * authldap_search(const gchar *query)
 			case LDAP_SUCCESS:
 				return ldap_res;
 				break;
+			case LDAP_NO_SUCH_OBJECT:
+				ldap_get_option(_ldap_conn, LDAP_OPT_DIAGNOSTIC_MESSAGE, &err_msg);
+				TRACE(TRACE_ERR,
+					  "LDAP error message (%d): %s. Please check LDAP config.",
+					  err, ldap_err2string(err)
+				);
+				return ldap_res;
+				break;
 			case LDAP_SERVER_DOWN:
 				ldap_get_option(_ldap_conn, LDAP_OPT_DIAGNOSTIC_MESSAGE, &err_msg);
 				TRACE(TRACE_WARNING, "LDAP gone away(%d): %s. Trying again(%d/%d). Error message: %s", err, ldap_err2string(err), c, c_tries, err_msg);
-				ldap_memfree(&err_msg);
+				if (err_msg) {
+					ldap_memfree(err_msg);
+					err_msg = NULL;
+				}
 				break;
 			default:
 				TRACE(TRACE_ERR, "LDAP error(%d): %s. Trying again (%d/%d).", err, ldap_err2string(err), c, c_tries);
@@ -317,18 +330,31 @@ void dm_ldap_freeresult(GList *entlist)
 	entlist = g_list_first(entlist);
 	while (entlist) {
 		fldlist = entlist->data;
+		fldlist = g_list_first(fldlist);
 		while(fldlist) {
 			attlist = fldlist->data;
+			attlist = g_list_first(attlist);
+			while(attlist) {
+				g_free(attlist->data);
+				if(!g_list_next(attlist))
+					break;
+				attlist = g_list_next(attlist);
+			}
+			attlist = g_list_first(attlist);
 			g_list_destroy(attlist);
-			if (! g_list_next(fldlist)) break;
+			if(!g_list_next(fldlist))
+				break;
 			fldlist = g_list_next(fldlist);
 		}
-		g_list_free(g_list_first(fldlist));
+		fldlist = g_list_first(fldlist);
+		g_list_free(fldlist);
 
-		if (! g_list_next(entlist)) break;
+		if(!g_list_next(entlist))
+			break;
 		entlist = g_list_next(entlist);
 	}
-	g_list_free(g_list_first(entlist));
+	entlist = g_list_first(entlist);
+	g_list_free(entlist);
 }
 
 static GList * dm_ldap_ent_get_values(GList *entlist)
@@ -341,17 +367,14 @@ static GList * dm_ldap_ent_get_values(GList *entlist)
 		while (fldlist) {
 			attlist = g_list_first(fldlist->data);
 			while (attlist) {
-				char *t = (gchar *)attlist->data;
+				gchar *t = (gchar *)attlist->data;
 				TRACE(TRACE_DEBUG,"value [%s]", t);
 				values = g_list_append_printf(values,"%s", t);
 
-				if (! g_list_next(attlist)) break;
 				attlist = g_list_next(attlist);
 			}
-			if (! g_list_next(fldlist)) break;
 			fldlist = g_list_next(fldlist);
 		}
-		if (! g_list_next(entlist)) break;
 		entlist = g_list_next(entlist);
 	}
 	return values;
@@ -361,14 +384,16 @@ static char *dm_ldap_get_filter(const gchar boolean, const gchar *attribute, GLi
 {
 	/* build user filter from objectclasses */
 	gchar *s;
-	GString *t = g_string_new("");
+	GString *t = NULL;
 	GString *q = g_string_new("");
 	GList *l = NULL;
 
 	values = g_list_first(values);
 	while (values) {
+		t = g_string_new("");
 		g_string_printf(t,"%s=%s", attribute, (char *)values->data);
 		l = g_list_append(l,g_strdup(t->str));
+		g_string_free(t, TRUE);
 		if (! g_list_next(values)) break;
 		values = g_list_next(values);
 	}
@@ -378,7 +403,7 @@ static char *dm_ldap_get_filter(const gchar boolean, const gchar *attribute, GLi
 
 	g_string_free(t,TRUE);
 	g_string_free(q,FALSE);
-	g_list_foreach(l,(GFunc)g_free,NULL);
+	g_list_free_full(g_steal_pointer (&l), g_free);
 
 	return s;
 }
@@ -754,7 +779,6 @@ int auth_getclientid(uint64_t user_idnr, uint64_t * client_idnr)
 	return TRUE;
 }
 
-
 int auth_getmaxmailsize(uint64_t user_idnr, uint64_t * maxmail_size)
 {
 	char *max_char;
@@ -814,6 +838,7 @@ GList * auth_get_known_users(void)
 	
 	query =  dm_ldap_get_filter('&',"objectClass",l);
 	entlist = __auth_get_every_match(query, fields);
+	g_list_free_full(g_steal_pointer (&l), g_free);
 	g_free(query);
 	
 	TRACE(TRACE_INFO, "found %d users", g_list_length(entlist));
@@ -839,6 +864,7 @@ GList * auth_get_known_aliases(void)
 	
 	query =  dm_ldap_get_filter('&',"objectClass",l);
 	entlist = __auth_get_every_match(query, fields);
+	g_list_free_full(g_steal_pointer (&l), g_free);
 	g_free(query);
 	
 	TRACE(TRACE_INFO, "found %d aliases", g_list_length(entlist));
@@ -857,10 +883,7 @@ GList * auth_get_known_aliases(void)
  * 
  * returns the number of occurences. 
  */
-
-
-	
-int auth_check_user_ext(const char *address, GList **userids, GList **fwds, int checks)
+int auth_check_user_ext(const char *userid, GList **userids, GList **fwds, int checks)
 {
 	int occurences = 0;
 	uint64_t id, *uid;
@@ -879,47 +902,53 @@ int auth_check_user_ext(const char *address, GList **userids, GList **fwds, int
 		return 0;
 	}
 
-	TRACE(TRACE_DEBUG, "checking user [%s] in alias table", address);
- 	if (strlen(_ldap_cfg.query_string)==0) {
- 		/* build a mail filter, with multiple attributes, if needed */
- 		GString *f = g_string_new(_ldap_cfg.field_mail);
- 		searchlist = g_string_split(f,",");
- 		g_string_free(f,TRUE);
- 	
- 		GString *t = g_string_new("");
- 		GString *q = g_string_new("");
- 		GList *l = NULL;
- 		searchlist = g_list_first(searchlist);
- 		while(searchlist) {
- 			g_string_printf(t,"%s=%s",(char *)searchlist->data,address);
- 			l = g_list_append(l,g_strdup(t->str));
- 			if(!g_list_next(searchlist))
- 				break;
- 			searchlist = g_list_next(searchlist);	
- 		}
-  
- 		t = g_list_join(l,")(");
- 		g_string_printf(q,"(|(%s))", t->str);
- 		query = q->str;
- 		g_string_free(t,TRUE);
- 		g_string_free(q,FALSE);
+	TRACE(TRACE_DEBUG, "checking user [%s] in ldap", userid);
+	if (strlen(_ldap_cfg.query_string)==0) {
+		/* build a mail filter, with multiple attributes, if needed */
+		GString *f = g_string_new(_ldap_cfg.field_mail);
+		searchlist = g_string_split(f,",");
+		g_string_free(f,TRUE);
+
+		GString *t = NULL;
+		GString *q = g_string_new("");
+		GList *l = NULL;
+		searchlist = g_list_first(searchlist);
+		while(searchlist) {
+			t = g_string_new("");
+			g_string_printf(t,"%s=%s",(char *)searchlist->data,userid);
+			l = g_list_append(l,g_strdup(t->str));
+			g_string_free(t, TRUE);
+			if (!g_list_next(searchlist))
+				break;
+			searchlist = g_list_next(searchlist);
+		}
+		searchlist = g_list_first(searchlist);
+		g_list_foreach(searchlist, (GFunc)g_string_free, NULL);
+		searchlist = g_list_first(searchlist);
+		g_list_free(searchlist);
+
+		t = g_list_join(l,")(");
+		g_string_printf(q,"(|(%s))", t->str);
+		query = q->str;
+		g_string_free(t,TRUE);
+		g_string_free(q,FALSE);
 		g_list_foreach(l,(GFunc)g_free,NULL);
 		g_list_free(l);
- 	} else {
- 		int i;
- 		GString *q = g_string_new("");
- 		for (i = 0; _ldap_cfg.query_string[i] != '\0'; i++) {
- 			if (_ldap_cfg.query_string[i]=='%' && _ldap_cfg.query_string[i+1] && _ldap_cfg.query_string[i+1]=='s') {
- 				g_string_append(q,address);
- 				i++;
- 			} else {
- 				g_string_append_c(q,_ldap_cfg.query_string[i]);
- 			}
- 		}	
- 		query = q->str;
- 		g_string_free(q,FALSE);
- 	}
- 
+	} else {
+		int i;
+		GString *q = g_string_new("");
+		for (i = 0; _ldap_cfg.query_string[i] != '\0'; i++) {
+			if (_ldap_cfg.query_string[i]=='%' && _ldap_cfg.query_string[i+1] && _ldap_cfg.query_string[i+1]=='s') {
+				g_string_append(q,userid);
+				i++;
+			} else {
+				g_string_append_c(q,_ldap_cfg.query_string[i]);
+			}
+		}
+		query = q->str;
+		g_string_free(q,FALSE);
+	}
+
 	TRACE(TRACE_DEBUG, "searching with query [%s], checks [%d]", query, checks);
 
 	entlist = __auth_get_every_match(query, fields);
@@ -931,19 +960,19 @@ int auth_check_user_ext(const char *address, GList **userids, GList **fwds, int
 			 * but checks needs to be bigger then 0 because
 			 * else it could be the first query failure */
 
-			id = strtoull(address, &endptr, 10);
+			id = strtoull(userid, &endptr, 10);
 			if (*endptr == 0) { /* numeric deliver-to --> this is a userid */
 				TRACE(TRACE_DEBUG, "adding [%" PRIu64 "] to userids", id);
 				uid = g_new0(uint64_t,1);
 				*uid = id;
 				*(GList **)userids = g_list_prepend(*(GList **)userids, uid);
 			} else {
-				TRACE(TRACE_DEBUG, "adding [%s] to forwards", address);
-				*(GList **)fwds = g_list_prepend(*(GList **)fwds, g_strdup(address));
+				TRACE(TRACE_DEBUG, "adding [%s] to forwards", userid);
+				*(GList **)fwds = g_list_prepend(*(GList **)fwds, g_strdup(userid));
 			}
 			return 1;
 		} else {
-			TRACE(TRACE_DEBUG, "user [%s] not in aliases table", address);
+			TRACE(TRACE_DEBUG, "user [%s] not in ldap aliases", userid);
 			dm_ldap_freeresult(entlist);
 			return 0;
 		}
@@ -956,18 +985,14 @@ int auth_check_user_ext(const char *address, GList **userids, GList **fwds, int
 		while(fldlist) {
 			attlist = g_list_first(fldlist->data);
 			while(attlist) {
-				attrvalue = (char *)attlist->data;
+				attrvalue = attlist->data;
 				occurences += auth_check_user_ext(attrvalue, userids, fwds, checks+1);
-				
-				if (! g_list_next(attlist))
-					break;
+
 				attlist = g_list_next(attlist);
 			}
-			if (! g_list_next(fldlist))
-				break;
 			fldlist = g_list_next(fldlist);
 		}
-		if (! g_list_next(entlist))
+		if (!g_list_next(entlist))
 			break;
 		entlist = g_list_next(entlist);
 	}
