@@ -2,7 +2,7 @@
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -335,25 +335,36 @@ int StartCliServer(ServerConfig_T * conf)
 static void reopen_logs_level(ServerConfig_T *conf, Trace_T level)
 {
 	int serr;
+	Field_T val;
 
 	if (mainReload) {
 		mainReload = 0;
 		TRACE(TRACE_INFO, "reopening log files");
 	}
 
-	if (fstdout) fclose(fstdout);
-	if (fstderr) fclose(fstderr);
+	config_get_value("logfile", "DBMAIL", val);
+	if (strncmp(val, "stderr", 6) == 0) {
+		TRACE(TRACE_INFO, "Keeping stdout and stderr open");
+	} else {
+		if (fstdout) fclose(fstdout);
+		if (fstderr) fclose(fstderr);
+	}
 	if (fnull) fclose(fnull);
 
 	SetTraceLevel(conf->service_name);
 	config_get_timeout(conf, conf->service_name);
 
-	if (! (fstdout = freopen(conf->log, "a", stdout))) {
-		serr = errno;
-		TRACE(level, "freopen failed on [%s] [%s]", conf->log, strerror(serr));
-	}
+	//if (! (fstdout = freopen(conf->log, "a", stdout))) {
+	//	serr = errno;
+	//	TRACE(level, "freopen failed on [%s] [%s]", conf->log, strerror(serr));
+	//}
 
-	if (! (fstderr = freopen(conf->error_log, "a", stderr))) {
+	//if (! (fstderr = freopen(conf->error_log, "a", stderr))) {
+	if (strncmp(val, "stderr", 6) == 0) {
+		TRACE(TRACE_INFO, "Keeping stdout and stderr open");
+		return;
+	}
+	if (! (fstderr = freopen(conf->log, "a", stderr))) {
 		serr = errno;
 		TRACE(level, "freopen failed on [%s] [%s]", conf->error_log, strerror(serr));
 	}
@@ -410,7 +421,9 @@ static int dm_bind_and_listen(int sock, struct sockaddr *saddr, socklen_t len, i
 	/* bind the address */
 	if ((bind(sock, saddr, len)) == -1) {
 		err = errno;
-		TRACE(TRACE_EMERG, "bind::error [%s]", strerror(err));
+		TRACE(TRACE_ERR, "Do you have permission to bind to address [%s:%s]?", hbuf, sbuf);
+		TRACE(TRACE_ERR, "bind::error [%s]", strerror(err));
+		TRACE(TRACE_EMERG, "Error creating %s socket [%d] on [%s:%s]", ssl?"ssl":"plain", sock, hbuf, sbuf);
 	}
 
 	if ((listen(sock, backlog)) == -1) {
@@ -768,7 +781,7 @@ int server_run(ServerConfig_T *conf)
 	assert(conf);
 	reopen_logs(conf);
 
- 	TRACE(TRACE_NOTICE, "starting main service loop for [%s]", conf->service_name);
+ 	TRACE(TRACE_NOTICE, "starting [%s] server_run", conf->service_name);
 
 	server_conf = conf;
 	if (db_connect()) {
@@ -782,8 +795,6 @@ int server_run(ServerConfig_T *conf)
 	}
 	srand((int) ((int) time(NULL) + (int) getpid()));
 
- 	TRACE(TRACE_NOTICE, "starting main service loop for [%s]", conf->service_name);
-
 	server_conf = conf;
 
 	evthread_use_pthreads();
@@ -848,6 +859,8 @@ int server_run(ServerConfig_T *conf)
 	
 	server_pidfile(conf);
 
+	TRACE(TRACE_NOTICE, "starting main service loop for [%s]", conf->service_name);
+
 	if (MATCH(conf->service_name, "IMAP"))
 		dm_queue_heartbeat();
 #ifdef HAVE_SYSTEMD
@@ -866,14 +879,15 @@ void server_showhelp(const char *name, const char *greeting) {
 	printf("%s\n", greeting);
 	printf("See the man page for more info.\n");
 
-        printf("\nCommon options for all DBMail daemons:\n");
-	printf("     -f file   specify an alternative config file\n");
-	printf("     -p file   specify an alternative runtime pidfile\n");
-	printf("     -n        stdin/stdout mode\n");
-	printf("     -D        foreground mode\n");
-	printf("     -v        verbose logging to syslog and stderr\n");
-	printf("     -V        show the version\n");
-	printf("     -h        show this help message\n");
+	printf("\nCommon options for all DBMail daemons:\n");
+	printf("  -f, --config file  specify an alternative config file\n");
+	printf("                     Default: %s\n", configFile);
+	printf("  -p, --pidfile file specify an alternative runtime pidfile\n");
+	printf("  -n, --inetd        stdin/stdout mode\n");
+	printf("  -D, --no-daemon    foreground mode\n");
+	printf("  -v, --verbose      verbose logging to syslog and stderr\n");
+	printf("  -V, --version      show the version\n");
+	printf("  -h, --help         show this help message\n");
 }
 
 /* Return values:
@@ -903,15 +917,28 @@ static void server_config_free(ServerConfig_T * config)
 int server_getopt(ServerConfig_T *config, const char *service, int argc, char *argv[])
 {
 	int opt;
+	int opt_index = 0;
 	memset(configFile, 0, sizeof(configFile));
 
 	g_strlcpy(configFile,DEFAULT_CONFIG_FILE, FIELDSIZE-1);
 
 	TRACE(TRACE_DEBUG, "checking command line options");
+	static struct option long_options[] = {
+		{"config", required_argument, NULL, 'f'},
+		{"pidfile", required_argument, NULL, 'p'},
+		{"inetd", no_argument, NULL, 'n'},
+		{"no-daemon", no_argument, NULL, 'D'},
+		{"version", no_argument, NULL, 'V'},
+		{"verbose", no_argument, NULL, 'v'},
+		{"help", no_argument, NULL, 'h'},
+		{0, 0, 0, 0}
+	};
 
 	/* get command-line options */
-	opterr = 0;		/* suppress error message from getopt() */
-	while ((opt = getopt(argc, argv, "vVhqnDf:p:s:")) != -1) {
+	opterr = 0; /* suppress error message from getopt() */
+	while ((opt = getopt_long(argc, argv,
+			"vVhqnDf:p:s:",
+			long_options, &opt_index)) != -1) {
 		switch (opt) {
 		case 'v':
 			config->log_verbose = 1;
@@ -931,7 +958,7 @@ int server_getopt(ServerConfig_T *config, const char *service, int argc, char *a
 			if (optarg && strlen(optarg) > 0)
 				config->pidFile = g_strdup(optarg);
 			else {
-				fprintf(stderr, "%s: -p requires a filename argument\n\n", argv[0]);
+				fprintf(stderr, "%s: --pidfile requires a filename argument\n\n", argv[0]);
 				return 1;
 			}
 			break;
@@ -939,7 +966,7 @@ int server_getopt(ServerConfig_T *config, const char *service, int argc, char *a
 			if (optarg && strlen(optarg) > 0) {
 				g_strlcpy(configFile, optarg, FIELDSIZE-1);
 			} else {
-				fprintf(stderr, "%s: -f requires a filename argument\n\n", argv[0]);
+				fprintf(stderr, "%s: --config requires a filename argument\n\n", argv[0]);
 				return 1;
 			}
 			break;
